@@ -15,11 +15,11 @@ class DataNode {
 public:
   virtual ~DataNode();
 
-  const blockstore::Key &key() const;
+  const blockstore::BlockId &blockId() const;
 
   uint8_t depth() const;
 
-  static cpputils::unique_ref<DataInnerNode> convertToNewInnerNode(cpputils::unique_ref<DataNode> node, const DataNode &first_child);
+  static cpputils::unique_ref<DataInnerNode> convertToNewInnerNode(cpputils::unique_ref<DataNode> node, const DataNodeLayout &layout, const DataNode &first_child);
 
   void flush() const;
 
