@@ -2,7 +2,7 @@
 #ifndef MESSMER_CPPUTILS_DATA_FIXEDSIZEDATA_H_
 #define MESSMER_CPPUTILS_DATA_FIXEDSIZEDATA_H_
 
-#include <cryptopp/hex.h>
+#include <vendor_cryptopp/hex.h>
 #include <string>
 #include <cstring>
 #include "../assert/assert.h"
@@ -34,10 +34,10 @@ public:
   template<size_t size> FixedSizeData<SIZE-size> drop() const;
 
 private:
-  FixedSizeData() {}
+  FixedSizeData(): _data() {}
   template<size_t _SIZE> friend class FixedSizeData;
 
-  unsigned char _data[BINARY_LENGTH];
+  std::array<unsigned char, BINARY_LENGTH> _data;
 };
 
 template<size_t SIZE> bool operator==(const FixedSizeData<SIZE> &lhs, const FixedSizeData<SIZE> &rhs);
@@ -51,7 +51,7 @@ template<size_t SIZE> constexpr size_t FixedSizeData<SIZE>::STRING_LENGTH;
 template<size_t SIZE>
 FixedSizeData<SIZE> FixedSizeData<SIZE>::Null() {
   FixedSizeData<SIZE> result;
-  std::memset(result._data, 0, BINARY_LENGTH);
+  std::memset(result._data.data(), 0, BINARY_LENGTH);
   return result;
 }
 
@@ -59,18 +59,20 @@ template<size_t SIZE>
 FixedSizeData<SIZE> FixedSizeData<SIZE>::FromString(const std::string &data) {
   ASSERT(data.size() == STRING_LENGTH, "Wrong string size for parsing FixedSizeData");
   FixedSizeData<SIZE> result;
-  CryptoPP::StringSource(data, true,
-    new CryptoPP::HexDecoder(
-      new CryptoPP::ArraySink(result._data, BINARY_LENGTH)
-    )
-  );
+  {
+    CryptoPP::StringSource _1(data, true,
+      new CryptoPP::HexDecoder(
+        new CryptoPP::ArraySink(result._data.data(), BINARY_LENGTH)
+      )
+    );
+  }
   return result;
 }
 
 template<size_t SIZE>
 std::string FixedSizeData<SIZE>::ToString() const {
   std::string result;
-  CryptoPP::ArraySource(_data, BINARY_LENGTH, true,
+  CryptoPP::ArraySource(_data.data(), BINARY_LENGTH, true,
     new CryptoPP::HexEncoder(
       new CryptoPP::StringSink(result)
     )
@@ -81,7 +83,7 @@ std::string FixedSizeData<SIZE>::ToString() const {
 
 template<size_t SIZE>
 const unsigned char *FixedSizeData<SIZE>::data() const {
-  return _data;
+  return _data.data();
 }
 
 template<size_t SIZE>
@@ -91,13 +93,13 @@ unsigned char *FixedSizeData<SIZE>::data() {
 
 template<size_t SIZE>
 void FixedSizeData<SIZE>::ToBinary(void *target) const {
-  std::memcpy(target, _data, BINARY_LENGTH);
+  std::memcpy(target, _data.data(), BINARY_LENGTH);
 }
 
 template<size_t SIZE>
 FixedSizeData<SIZE> FixedSizeData<SIZE>::FromBinary(const void *source) {
   FixedSizeData<SIZE> result;
-  std::memcpy(result._data, source, BINARY_LENGTH);
+  std::memcpy(result._data.data(), source, BINARY_LENGTH);
   return result;
 }
 
@@ -105,7 +107,7 @@ template<size_t SIZE> template<size_t size>
 FixedSizeData<size> FixedSizeData<SIZE>::take() const {
   static_assert(size <= SIZE, "Out of bounds");
   FixedSizeData<size> result;
-  std::memcpy(result._data, _data, size);
+  std::memcpy(result._data.data(), _data.data(), size);
   return result;
 }
 
@@ -113,7 +115,7 @@ template<size_t SIZE> template<size_t size>
 FixedSizeData<SIZE-size> FixedSizeData<SIZE>::drop() const {
   static_assert(size <= SIZE, "Out of bounds");
   FixedSizeData<SIZE-size> result;
-  std::memcpy(result._data, _data+size, SIZE-size);
+  std::memcpy(result._data.data(), _data.data()+size, SIZE-size);
   return result;
 }
 
