@@ -1,19 +1,41 @@
 // Base version taken from https://techoverflow.net/blog/2013/03/15/c-simple-http-download-using-libcurl-easy-api/
 
+#if !defined(_MSC_VER)
+
 #include "CurlHttpClient.h"
 #include <sstream>
 #include <iostream>
-#include <curl/easy.h>
 
 using boost::none;
 using boost::optional;
 using std::string;
 using std::ostringstream;
+using std::mutex;
+using std::unique_lock;
 
 namespace cpputils {
 
+	mutex CurlHttpClient::CurlInitializerRAII::_mutex;
+	uint32_t CurlHttpClient::CurlInitializerRAII::_refcount = 0;
+
+	CurlHttpClient::CurlInitializerRAII::CurlInitializerRAII() {
+		unique_lock<mutex> lock(_mutex);
+		if (0 == _refcount) {
+			curl_global_init(CURL_GLOBAL_ALL);
+		}
+		_refcount += 1;
+	}
+
+	CurlHttpClient::CurlInitializerRAII::~CurlInitializerRAII() {
+		unique_lock<mutex> lock(_mutex);
+		_refcount -= 1;
+		if (0 == _refcount) {
+			curl_global_cleanup();
+		}
+	}
+
     size_t CurlHttpClient::write_data(void *ptr, size_t size, size_t nmemb, ostringstream *stream) {
-        stream->write((const char *) ptr, size * nmemb);
+        stream->write(static_cast<const char *>(ptr), size * nmemb);
         return size * nmemb;
     }
 
@@ -25,7 +47,7 @@ namespace cpputils {
         curl_easy_cleanup(curl);
     }
 
-    optional <string> CurlHttpClient::get(const string &url, optional<long> timeoutMsec) {
+    string CurlHttpClient::get(const string &url, optional<long> timeoutMsec) {
         curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
         // example.com is redirected, so we tell libcurl to follow redirection
         curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1L);
@@ -41,9 +63,11 @@ namespace cpputils {
         CURLcode res = curl_easy_perform(curl);
         // Check for errors
         if (res != CURLE_OK) {
-            return none;
+			throw std::runtime_error("Curl Error " + std::to_string(res) + ": " + curl_easy_strerror(res));
         }
         return out.str();
     }
 
 }
+
+#endif
