@@ -1,8 +1,6 @@
 #include "Cli.h"
 
-#include <blockstore/implementations/ondisk/OnDiskBlockStore.h>
-#include <blockstore/implementations/inmemory/InMemoryBlockStore.h>
-#include <blockstore/implementations/inmemory/InMemoryBlock.h>
+#include <blockstore/implementations/ondisk/OnDiskBlockStore2.h>
 #include <cmath>
 #include <cstdio>
 #include <cstdlib>
@@ -14,6 +12,7 @@
 #include <cpp-utils/io/DontEchoStdinToStdoutRAII.h>
 #include <cryfs/filesystem/CryDevice.h>
 #include <cryfs/config/CryConfigLoader.h>
+#include <cryfs/config/CryPasswordBasedKeyProvider.h>
 #include "program_options/Parser.h"
 #include <boost/filesystem.hpp>
 
@@ -23,45 +22,42 @@
 #include "VersionChecker.h"
 #include <gitversion/VersionCompare.h>
 #include <cpp-utils/io/NoninteractiveConsole.h>
+#include <cryfs/localstate/LocalStateDir.h>
+#include <cryfs/localstate/BasedirMetadata.h>
 #include "Environment.h"
 #include <cryfs/CryfsException.h>
+#include <cpp-utils/thread/debugging.h>
 
 //TODO Many functions accessing the ProgramOptions object. Factor out into class that stores it as a member.
 //TODO Factor out class handling askPassword
 
+using namespace cryfs_cli;
 using namespace cryfs;
 namespace bf = boost::filesystem;
 using namespace cpputils::logging;
 
-using blockstore::ondisk::OnDiskBlockStore;
-using blockstore::inmemory::InMemoryBlockStore;
+using blockstore::ondisk::OnDiskBlockStore2;
 using program_options::ProgramOptions;
 
 using cpputils::make_unique_ref;
-using cpputils::Random;
 using cpputils::NoninteractiveConsole;
 using cpputils::TempFile;
 using cpputils::RandomGenerator;
 using cpputils::unique_ref;
+using cpputils::SCrypt;
 using cpputils::SCryptSettings;
 using cpputils::Console;
 using cpputils::HttpClient;
-using cpputils::DontEchoStdinToStdoutRAII;
-using std::cin;
 using std::cout;
 using std::string;
 using std::endl;
-using std::vector;
 using std::shared_ptr;
 using std::make_shared;
 using std::unique_ptr;
 using std::make_unique;
 using std::function;
-using std::make_shared;
 using boost::optional;
 using boost::none;
-using boost::chrono::duration;
-using boost::chrono::duration_cast;
 using boost::chrono::minutes;
 using boost::chrono::milliseconds;
 using cpputils::dynamic_pointer_move;
@@ -72,10 +68,10 @@ using gitversion::VersionCompare;
 //TODO Replace ASSERTs with other error handling when it is not a programming error but an environment influence (e.g. a block is missing)
 //TODO Can we improve performance by setting compiler parameter -maes for scrypt?
 
-namespace cryfs {
+namespace cryfs_cli {
 
-    Cli::Cli(RandomGenerator &keyGenerator, const SCryptSettings &scryptSettings, shared_ptr<Console> console, shared_ptr<HttpClient> httpClient):
-            _keyGenerator(keyGenerator), _scryptSettings(scryptSettings), _console(), _httpClient(httpClient), _noninteractive(false) {
+    Cli::Cli(RandomGenerator &keyGenerator, const SCryptSettings &scryptSettings, shared_ptr<Console> console):
+            _keyGenerator(keyGenerator), _scryptSettings(scryptSettings), _console(), _noninteractive(false), _idleUnmounter(none), _device(none) {
         _noninteractive = Environment::isNoninteractive();
         if (_noninteractive) {
             _console = make_shared<NoninteractiveConsole>(console);
@@ -84,7 +80,7 @@ namespace cryfs {
         }
     }
 
-    void Cli::_showVersion() {
+    void Cli::_showVersion(unique_ref<HttpClient> httpClient) {
         cout << "CryFS Version " << gitversion::VersionString() << endl;
         if (gitversion::IsDevVersion()) {
             cout << "WARNING! This is a development version based on git commit " << gitversion::GitCommitId() <<
@@ -101,7 +97,7 @@ namespace cryfs {
         } else if (Environment::isNoninteractive()) {
             cout << "Automatic checking for security vulnerabilities and updates is disabled in noninteractive mode." << endl;
         } else {
-            _checkForUpdates();
+            _checkForUpdates(std::move(httpClient));
         }
 #else
 # warning Update checks are disabled. The resulting executable will not go online to check for newer versions or known security vulnerabilities.
@@ -109,8 +105,8 @@ namespace cryfs {
         cout << endl;
     }
 
-    void Cli::_checkForUpdates() {
-        VersionChecker versionChecker(_httpClient);
+    void Cli::_checkForUpdates(unique_ref<HttpClient> httpClient) {
+        VersionChecker versionChecker(httpClient.get());
         optional<string> newestVersion = versionChecker.newestVersion();
         if (newestVersion == none) {
             cout << "Could not check for updates." << endl;
@@ -131,34 +127,39 @@ namespace cryfs {
         return true;
     }
 
-    string Cli::_askPasswordForExistingFilesystem() {
-        string password = _askPasswordFromStdin("Password: ");
-        while (!_checkPassword(password)) {
-            password = _askPasswordFromStdin("Password: ");
-        }
-        return password;
+    function<string()> Cli::_askPasswordForExistingFilesystem(std::shared_ptr<cpputils::Console> console) {
+        return [console] () {
+            string password = console->askPassword("Password: ");
+            while (!_checkPassword(password)) {
+                password = console->askPassword("Password: ");
+            }
+            return password;
+        };
     };
 
-    string Cli::_askPasswordForNewFilesystem() {
-        string password;
-        bool again = false;
-        do {
-            password = _askPasswordFromStdin("Password: ");
-            if (!_checkPassword(password)) {
-                again = true;
-                continue;
-            }
-            if (!_confirmPassword(password)) {
-                again = true;
-                continue;
-            }
-            again = false;
-        } while(again);
-        return password;
+    function<string()> Cli::_askPasswordForNewFilesystem(std::shared_ptr<cpputils::Console> console) {
+        //TODO Ask confirmation if using insecure password (<8 characters)
+        return [console] () {
+            string password;
+            bool again = false;
+            do {
+                password = console->askPassword("Password: ");
+                if (!_checkPassword(password)) {
+                    again = true;
+                    continue;
+                }
+                if (!_confirmPassword(console.get(), password)) {
+                    again = true;
+                    continue;
+                }
+                again = false;
+            } while (again);
+            return password;
+        };
     }
 
-    bool Cli::_confirmPassword(const string &password) {
-        string confirmPassword = _askPasswordFromStdin("Confirm Password: ");
+    bool Cli::_confirmPassword(cpputils::Console* console, const string &password) {
+        string confirmPassword = console->askPassword("Confirm Password: ");
         if (password != confirmPassword) {
             std::cout << "Passwords don't match" << std::endl;
             return false;
@@ -166,29 +167,15 @@ namespace cryfs {
         return true;
     }
 
-    string Cli::_askPasswordNoninteractive() {
+    function<string()> Cli::_askPasswordNoninteractive(std::shared_ptr<cpputils::Console> console) {
         //TODO Test
-        string password = _askPasswordFromStdin("Password: ");
-        if (!_checkPassword(password)) {
-            throw CryfsException("Invalid password. Password cannot be empty.", ErrorCode::EmptyPassword);
-        }
-        return password;
-    }
-
-    string Cli::_askPasswordFromStdin(const string &prompt) {
-        DontEchoStdinToStdoutRAII _stdin_input_is_hidden_as_long_as_this_is_in_scope;
-
-        std::cout << prompt << std::flush;
-        string result;
-        std::getline(cin, result);
-        std::cout << std::endl;
-
-        //Remove trailing newline
-        if (result[result.size()-1] == '\n') {
-            result.resize(result.size()-1);
-        }
-
-        return result;
+        return [console] () {
+            string password = console->askPassword("Password: ");
+            if (!_checkPassword(password)) {
+                throw CryfsException("Invalid password. Password cannot be empty.", ErrorCode::EmptyPassword);
+            }
+            return password;
+        };
     }
 
     bf::path Cli::_determineConfigFile(const ProgramOptions &options) {
@@ -199,60 +186,101 @@ namespace cryfs {
         return *configFile;
     }
 
-    CryConfigFile Cli::_loadOrCreateConfig(const ProgramOptions &options) {
+    void Cli::_checkConfigIntegrity(const bf::path& basedir, const LocalStateDir& localStateDir, const CryConfigFile& config, bool allowReplacedFilesystem) {
+        auto basedirMetadata = BasedirMetadata::load(localStateDir);
+        if (!allowReplacedFilesystem && !basedirMetadata.filesystemIdForBasedirIsCorrect(basedir, config.config()->FilesystemId())) {
+          if (!_console->askYesNo("The filesystem id in the config file is different to the last time we loaded a filesystem from this basedir. This can be genuine if you replaced the filesystem with a different one. If you didn't do that, it is possible that an attacker did. Do you want to continue loading the file system?", false)) {
+            throw CryfsException(
+                "The filesystem id in the config file is different to the last time we loaded a filesystem from this basedir.", ErrorCode::FilesystemIdChanged);
+          }
+        }
+        // Update local state (or create it if it didn't exist yet)
+        basedirMetadata.updateFilesystemIdForBasedir(basedir, config.config()->FilesystemId());
+        basedirMetadata.save();
+    }
+
+    CryConfigLoader::ConfigLoadResult Cli::_loadOrCreateConfig(const ProgramOptions &options, const LocalStateDir& localStateDir) {
         auto configFile = _determineConfigFile(options);
-        auto config = _loadOrCreateConfigFile(configFile, options.cipher(), options.blocksizeBytes(),
-                                              options.allowFilesystemUpgrade());
+        auto config = _loadOrCreateConfigFile(std::move(configFile), localStateDir, options.cipher(), options.blocksizeBytes(), options.allowFilesystemUpgrade(), options.missingBlockIsIntegrityViolation(), options.allowReplacedFilesystem());
         if (config == none) {
           throw CryfsException("Could not load config file. Did you enter the correct password?", ErrorCode::WrongPassword);
         }
+        _checkConfigIntegrity(options.baseDir(), localStateDir, config->configFile, options.allowReplacedFilesystem());
         return std::move(*config);
     }
 
-    optional<CryConfigFile> Cli::_loadOrCreateConfigFile(const bf::path &configFilePath, const optional<string> &cipher, const optional<uint32_t> &blocksizeBytes, bool allowFilesystemUpgrade) {
-        if (_noninteractive) {
-            return CryConfigLoader(_console, _keyGenerator, _scryptSettings,
-                                   &Cli::_askPasswordNoninteractive,
-                                   &Cli::_askPasswordNoninteractive,
-                                   cipher, blocksizeBytes).loadOrCreate(configFilePath, allowFilesystemUpgrade);
-        } else {
-            return CryConfigLoader(_console, _keyGenerator, _scryptSettings,
-                                   &Cli::_askPasswordForExistingFilesystem,
-                                   &Cli::_askPasswordForNewFilesystem,
-                                   cipher, blocksizeBytes).loadOrCreate(configFilePath, allowFilesystemUpgrade);
-        }
+    optional<CryConfigLoader::ConfigLoadResult> Cli::_loadOrCreateConfigFile(bf::path configFilePath, LocalStateDir localStateDir, const optional<string> &cipher, const optional<uint32_t> &blocksizeBytes, bool allowFilesystemUpgrade, const optional<bool> &missingBlockIsIntegrityViolation, bool allowReplacedFilesystem) {
+        // TODO Instead of passing in _askPasswordXXX functions to KeyProvider, only pass in console and move logic to the key provider,
+        //      for example by having a separate CryPasswordBasedKeyProvider / CryNoninteractivePasswordBasedKeyProvider.
+        auto keyProvider = make_unique_ref<CryPasswordBasedKeyProvider>(
+          _console,
+          _noninteractive ? Cli::_askPasswordNoninteractive(_console) : Cli::_askPasswordForExistingFilesystem(_console),
+          _noninteractive ? Cli::_askPasswordNoninteractive(_console) : Cli::_askPasswordForNewFilesystem(_console),
+          make_unique_ref<SCrypt>(_scryptSettings)
+        );
+        return CryConfigLoader(_console, _keyGenerator, std::move(keyProvider), std::move(localStateDir),
+                               cipher, blocksizeBytes, missingBlockIsIntegrityViolation).loadOrCreate(std::move(configFilePath), allowFilesystemUpgrade, allowReplacedFilesystem);
     }
 
-    void Cli::_runFilesystem(const ProgramOptions &options) {
+    void Cli::_runFilesystem(const ProgramOptions &options, std::function<void()> onMounted) {
         try {
-            auto blockStore = make_unique_ref<OnDiskBlockStore>(options.baseDir());
-            auto config = _loadOrCreateConfig(options);
-            CryDevice device(std::move(config), std::move(blockStore));
-            _sanityCheckFilesystem(&device);
-            fspp::FilesystemImpl fsimpl(&device);
-            fspp::fuse::Fuse fuse(&fsimpl, "cryfs", "cryfs@"+options.baseDir().native());
+            LocalStateDir localStateDir(Environment::localStateDir());
+            auto blockStore = make_unique_ref<OnDiskBlockStore2>(options.baseDir());
+            auto config = _loadOrCreateConfig(options, localStateDir);
+            unique_ptr<fspp::fuse::Fuse> fuse = nullptr;
+            bool stoppedBecauseOfIntegrityViolation = false;
+
+            auto onIntegrityViolation = [&fuse, &stoppedBecauseOfIntegrityViolation] () {
+              if (fuse.get() != nullptr) {
+                LOG(ERR, "Integrity violation detected. Unmounting.");
+                stoppedBecauseOfIntegrityViolation = true;
+                fuse->stop();
+              } else {
+                // Usually on an integrity violation, the file system is unmounted.
+                // Here, the file system isn't initialized yet, i.e. we failed in the initial steps when
+                // setting up _device before running initFilesystem.
+                // We can't unmount a not-mounted file system, but we can make sure it doesn't get mounted.
+                throw CryfsException("Integrity violation detected. Unmounting.", ErrorCode::IntegrityViolation);
+              }
+            };
+            const bool missingBlockIsIntegrityViolation = config.configFile.config()->missingBlockIsIntegrityViolation();
+            _device = optional<unique_ref<CryDevice>>(make_unique_ref<CryDevice>(std::move(config.configFile), std::move(blockStore), std::move(localStateDir), config.myClientId,
+                                                                                 options.allowIntegrityViolations(), missingBlockIsIntegrityViolation, std::move(onIntegrityViolation)));
+            _sanityCheckFilesystem(_device->get());
+
+            auto initFilesystem = [&] (fspp::fuse::Fuse *fs){
+                ASSERT(_device != none, "File system not ready to be initialized. Was it already initialized before?");
+
+                //TODO Test auto unmounting after idle timeout
+                const boost::optional<double> idle_minutes = options.unmountAfterIdleMinutes();
+                _idleUnmounter = _createIdleCallback(idle_minutes, [fs, idle_minutes] {
+                    LOG(INFO, "Unmounting because file system was idle for {} minutes", *idle_minutes);
+                    fs->stop();
+                });
+                if (_idleUnmounter != none) {
+                    (*_device)->onFsAction(std::bind(&CallAfterTimeout::resetTimer, _idleUnmounter->get()));
+                }
+
+                return make_shared<fspp::FilesystemImpl>(std::move(*_device));
+            };
+
+            fuse = make_unique<fspp::fuse::Fuse>(initFilesystem, std::move(onMounted), "cryfs", "cryfs@" + options.baseDir().string());
 
             _initLogfile(options);
 
-            //TODO Test auto unmounting after idle timeout
-            //TODO This can fail due to a race condition if the filesystem isn't started yet (e.g. passing --unmount-idle 0").
-            auto idleUnmounter = _createIdleCallback(options.unmountAfterIdleMinutes(), [&fuse] {fuse.stop();});
-            if (idleUnmounter != none) {
-                device.onFsAction(std::bind(&CallAfterTimeout::resetTimer, idleUnmounter->get()));
-            }
+          std::cout << "\nMounting filesystem. To unmount, call:\n$ cryfs-unmount " << options.mountDir() << "\n"
+                    << std::endl;
+          fuse->run(options.mountDir(), options.fuseOptions());
 
-#ifdef __APPLE__
-            std::cout << "\nMounting filesystem. To unmount, call:\n$ umount " << options.mountDir() << "\n" << std::endl;
-#else
-            std::cout << "\nMounting filesystem. To unmount, call:\n$ fusermount -u " << options.mountDir() << "\n" << std::endl;
-#endif
-            fuse.run(options.mountDir(), options.fuseOptions());
+          if (stoppedBecauseOfIntegrityViolation) {
+              throw CryfsException("Integrity violation detected. Unmounting.", ErrorCode::IntegrityViolation);
+          }
         } catch (const CryfsException &e) {
             throw; // CryfsException is only thrown if setup goes wrong. Throw it through so that we get the correct process exit code.
         } catch (const std::exception &e) {
-            LOG(ERROR, "Crashed: {}", e.what());
+            LOG(ERR, "Crashed: {}", e.what());
         } catch (...) {
-            LOG(ERROR, "Crashed");
+            LOG(ERR, "Crashed");
         }
     }
 
@@ -273,8 +301,8 @@ namespace cryfs {
         if (minutes == none) {
             return none;
         }
-        uint64_t millis = std::round(60000 * (*minutes));
-        return make_unique_ref<CallAfterTimeout>(milliseconds(millis), callback);
+        uint64_t millis = std::llround(60000 * (*minutes));
+        return make_unique_ref<CallAfterTimeout>(milliseconds(millis), callback, "idlecallback");
     }
 
     void Cli::_initLogfile(const ProgramOptions &options) {
@@ -282,18 +310,25 @@ namespace cryfs {
         //TODO Test that --logfile parameter works. Should be: file if specified, otherwise stderr if foreground, else syslog.
         if (options.logFile() != none) {
             cpputils::logging::setLogger(
-                spdlog::create<spdlog::sinks::simple_file_sink<std::mutex>>("cryfs", options.logFile()->native()));
+                spdlog::create<spdlog::sinks::simple_file_sink<std::mutex>>("cryfs", options.logFile()->string()));
         } else if (options.foreground()) {
             cpputils::logging::setLogger(spdlog::stderr_logger_mt("cryfs"));
         } else {
-            cpputils::logging::setLogger(spdlog::syslog_logger("cryfs", "cryfs", LOG_PID));
+            cpputils::logging::setLogger(cpputils::logging::system_logger("cryfs"));
         }
     }
 
-    void Cli::_sanityChecks(const ProgramOptions &options) {
-        _checkDirAccessible(options.baseDir(), "base directory", ErrorCode::InaccessibleBaseDir);
-        _checkDirAccessible(options.mountDir(), "mount directory", ErrorCode::InaccessibleMountDir);
-        _checkMountdirDoesntContainBasedir(options);
+	void Cli::_sanityChecks(const ProgramOptions &options) {
+		_checkDirAccessible(bf::absolute(options.baseDir()), "base directory", ErrorCode::InaccessibleBaseDir);
+
+		if (!options.mountDirIsDriveLetter()) {
+			_checkDirAccessible(options.mountDir(), "mount directory", ErrorCode::InaccessibleMountDir);
+			_checkMountdirDoesntContainBasedir(options);
+		} else {
+			if (bf::exists(options.mountDir())) {
+				throw CryfsException("Drive " + options.mountDir().string() + " already exists.", ErrorCode::InaccessibleMountDir);
+			}
+		}
     }
 
     void Cli::_checkDirAccessible(const bf::path &dir, const std::string &name, ErrorCode errorCode) {
@@ -364,17 +399,18 @@ namespace cryfs {
         return false;
     }
 
-    int Cli::main(int argc, const char *argv[]) {
-        cpputils::showBacktraceOnSigSegv();
+    int Cli::main(int argc, const char **argv, unique_ref<HttpClient> httpClient, std::function<void()> onMounted) {
+        cpputils::showBacktraceOnCrash();
+        cpputils::set_thread_name("cryfs");
 
         try {
-            _showVersion();
+            _showVersion(std::move(httpClient));
             ProgramOptions options = program_options::Parser(argc, argv).parse(CryCiphers::supportedCipherNames());
             _sanityChecks(options);
-            _runFilesystem(options);
+            _runFilesystem(options, std::move(onMounted));
         } catch (const CryfsException &e) {
-            if (e.errorCode() != ErrorCode::Success) {
-              std::cerr << "Error: " << e.what() << std::endl;
+            if (e.what() != string()) {
+              std::cerr << "Error " << static_cast<int>(e.errorCode()) << ": " << e.what() << std::endl;
             }
             return exitCode(e.errorCode());
         } catch (const std::runtime_error &e) {
