@@ -14,9 +14,11 @@ namespace cryfs {
 
         std::string askCipher();
         uint32_t askBlocksizeBytes();
+        bool askMissingBlockIsIntegrityViolation();
 
         static constexpr const char *DEFAULT_CIPHER = "aes-256-gcm";
-        static constexpr uint32_t DEFAULT_BLOCKSIZE_BYTES = 32 * 1024; // 32KB
+        static constexpr uint32_t DEFAULT_BLOCKSIZE_BYTES = 16 * 1024; // 16KB
+        static constexpr uint32_t DEFAULT_MISSINGBLOCKISINTEGRITYVIOLATION = false;
 
     private:
 
@@ -25,6 +27,7 @@ namespace cryfs {
         std::string _askCipher() const;
         bool _showWarningForCipherAndReturnIfOk(const std::string &cipherName) const;
         uint32_t _askBlocksizeBytes() const;
+        bool _askMissingBlockIsIntegrityViolation() const;
 
         std::shared_ptr<cpputils::Console> _console;
         boost::optional<bool> _useDefaultSettings;
