@@ -12,19 +12,19 @@ class FileBlobRef final: public FsBlobRef {
 public:
     FileBlobRef(cachingfsblobstore::FileBlobRef *base) : _base(base) {}
 
-    void resize(off_t size) {
+    void resize(fspp::num_bytes_t size) {
         return _base->resize(size);
     }
 
-    off_t size() const {
+    fspp::num_bytes_t size() const {
         return _base->size();
     }
 
-    ssize_t read(void *target, uint64_t offset, uint64_t count) const {
+    fspp::num_bytes_t read(void *target, fspp::num_bytes_t offset, fspp::num_bytes_t count) const {
         return _base->read(target, offset, count);
     }
 
-    void write(const void *source, uint64_t offset, uint64_t count) {
+    void write(const void *source, fspp::num_bytes_t offset, fspp::num_bytes_t count) {
         return _base->write(source, offset, count);
     }
 
@@ -32,14 +32,22 @@ public:
         return _base->flush();
     }
 
-    const blockstore::Key &key() const {
-        return _base->key();
+    const blockstore::BlockId &blockId() const override {
+        return _base->blockId();
     }
 
-    off_t lstat_size() const {
+    fspp::num_bytes_t lstat_size() const override {
         return _base->lstat_size();
     }
 
+    const blockstore::BlockId &parentPointer() const override {
+        return _base->parentPointer();
+    }
+
+    void setParentPointer(const blockstore::BlockId &parentId) override {
+        return _base->setParentPointer(parentId);
+    }
+
 private:
     cachingfsblobstore::FileBlobRef *_base;
 
