@@ -9,14 +9,14 @@ class FsppDeviceTest_Timestamps: public FsppNodeTest<ConcreteFileSystemTestFixtu
 public:
   void Test_Load_While_Loaded() {
     auto node = this->CreateNode("/mynode");
-    auto operation = [this, &node] () {
+    auto operation = [this] () {
         this->device->Load("/mynode");
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/mynode", operation, {this->ExpectDoesntUpdateAnyTimestamps});
   }
 
   void Test_Load_While_Not_Loaded() {
-    struct stat oldStat;
+    fspp::Node::stat_info oldStat{};
     {
         auto node = this->CreateNode("/mynode");
         oldStat = this->stat(*node);
@@ -28,10 +28,10 @@ public:
     auto node = this->device->Load("/mynode");
 
     //Test that timestamps didn't change
-    struct stat newStat = this->stat(*node.value());
-    EXPECT_EQ(oldStat.st_atim, newStat.st_atim);
-    EXPECT_EQ(oldStat.st_mtim, newStat.st_mtim);
-    EXPECT_EQ(oldStat.st_ctim, newStat.st_ctim);
+    fspp::Node::stat_info newStat = this->stat(*node.value());
+    EXPECT_EQ(oldStat.atime, newStat.atime);
+    EXPECT_EQ(oldStat.mtime, newStat.mtime);
+    EXPECT_EQ(oldStat.ctime, newStat.ctime);
   }
 };
 
