@@ -7,10 +7,10 @@
 template<class ConcreteFileSystemTestFixture>
 class FsppFileTest_Timestamps: public TimestampTestUtils<ConcreteFileSystemTestFixture> {
 public:
-    cpputils::unique_ref<fspp::File> CreateFileWithSize(const boost::filesystem::path &path, off_t size) {
+    cpputils::unique_ref<fspp::File> CreateFileWithSize(const boost::filesystem::path &path, fspp::num_bytes_t size) {
         auto file = this->CreateFile(path);
         file->truncate(size);
-        assert(this->stat(*this->Load(path)).st_size == size);
+        assert(this->stat(*this->Load(path)).size == size);
         return file;
     }
 };
@@ -19,7 +19,7 @@ TYPED_TEST_CASE_P(FsppFileTest_Timestamps);
 TYPED_TEST_P(FsppFileTest_Timestamps, open_nomode) {
     auto file = this->CreateFile("/myfile");
     auto operation = [&file] () {
-        file->open(0);
+        file->open(fspp::openflags_t(0));
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/myfile", operation, {this->ExpectDoesntUpdateAnyTimestamps});
 }
@@ -27,7 +27,7 @@ TYPED_TEST_P(FsppFileTest_Timestamps, open_nomode) {
 TYPED_TEST_P(FsppFileTest_Timestamps, open_rdonly) {
     auto file = this->CreateFile("/myfile");
     auto operation = [&file] () {
-        file->open(O_RDONLY);
+        file->open(fspp::openflags_t::RDONLY());
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/myfile", operation, {this->ExpectDoesntUpdateAnyTimestamps});
 }
@@ -35,55 +35,55 @@ TYPED_TEST_P(FsppFileTest_Timestamps, open_rdonly) {
 TYPED_TEST_P(FsppFileTest_Timestamps, open_wronly) {
     auto file = this->CreateFile("/myfile");
     auto operation = [&file] () {
-        file->open(O_WRONLY);
+        file->open(fspp::openflags_t::WRONLY());
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/myfile", operation, {this->ExpectDoesntUpdateAnyTimestamps});
 }
-
+	
 TYPED_TEST_P(FsppFileTest_Timestamps, open_rdwr) {
     auto file = this->CreateFile("/myfile");
     auto operation = [&file] () {
-        file->open(O_RDWR);
+        file->open(fspp::openflags_t::RDWR());
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/myfile", operation, {this->ExpectDoesntUpdateAnyTimestamps});
 }
 
 TYPED_TEST_P(FsppFileTest_Timestamps, truncate_empty_to_empty) {
-    auto file = this->CreateFileWithSize("/myfile", 0);
+    auto file = this->CreateFileWithSize("/myfile", fspp::num_bytes_t(0));
     auto operation = [&file] () {
-        file->truncate(0);
+        file->truncate(fspp::num_bytes_t(0));
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/myfile", operation, {this->ExpectDoesntUpdateAccessTimestamp, this->ExpectUpdatesModificationTimestamp, this->ExpectUpdatesMetadataTimestamp});
 }
 
 TYPED_TEST_P(FsppFileTest_Timestamps, truncate_empty_to_nonempty) {
-    auto file = this->CreateFileWithSize("/myfile", 0);
+    auto file = this->CreateFileWithSize("/myfile", fspp::num_bytes_t(0));
     auto operation = [&file] () {
-        file->truncate(10);
+        file->truncate(fspp::num_bytes_t(10));
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/myfile", operation, {this->ExpectDoesntUpdateAccessTimestamp, this->ExpectUpdatesModificationTimestamp, this->ExpectUpdatesMetadataTimestamp});
 }
 
 TYPED_TEST_P(FsppFileTest_Timestamps, truncate_nonempty_to_empty) {
-    auto file = this->CreateFileWithSize("/myfile", 10);
+    auto file = this->CreateFileWithSize("/myfile", fspp::num_bytes_t(10));
     auto operation = [&file] () {
-        file->truncate(0);
+        file->truncate(fspp::num_bytes_t(0));
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/myfile", operation, {this->ExpectDoesntUpdateAccessTimestamp, this->ExpectUpdatesModificationTimestamp, this->ExpectUpdatesMetadataTimestamp});
 }
 
 TYPED_TEST_P(FsppFileTest_Timestamps, truncate_nonempty_to_nonempty_shrink) {
-    auto file = this->CreateFileWithSize("/myfile", 10);
+    auto file = this->CreateFileWithSize("/myfile", fspp::num_bytes_t(10));
     auto operation = [&file] () {
-        file->truncate(5);
+        file->truncate(fspp::num_bytes_t(5));
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/myfile", operation, {this->ExpectDoesntUpdateAccessTimestamp, this->ExpectUpdatesModificationTimestamp, this->ExpectUpdatesMetadataTimestamp});
 }
 
 TYPED_TEST_P(FsppFileTest_Timestamps, truncate_nonempty_to_nonempty_grow) {
-    auto file = this->CreateFileWithSize("/myfile", 10);
+    auto file = this->CreateFileWithSize("/myfile", fspp::num_bytes_t(10));
     auto operation = [&file] () {
-        file->truncate(20);
+        file->truncate(fspp::num_bytes_t(20));
     };
     this->EXPECT_OPERATION_UPDATES_TIMESTAMPS_AS("/myfile", operation, {this->ExpectDoesntUpdateAccessTimestamp, this->ExpectUpdatesModificationTimestamp, this->ExpectUpdatesMetadataTimestamp});
 }
