@@ -2,32 +2,66 @@
 #include <memory>
 #include <cassert>
 
-#include "FuseErrnoException.h"
+#include "../fs_interface/FuseErrnoException.h"
 #include "Filesystem.h"
 #include <iostream>
 #include <cpp-utils/assert/assert.h>
 #include <cpp-utils/logging/logging.h>
+#include <cpp-utils/process/subprocess.h>
+#include <cpp-utils/thread/debugging.h>
 #include <csignal>
+#include "InvalidFilesystem.h"
+#include <codecvt>
+
+#if defined(_MSC_VER)
+#include <codecvt>
+#include <dokan/dokan.h>
+#endif
 
 using std::vector;
 using std::string;
 
 namespace bf = boost::filesystem;
 using namespace cpputils::logging;
+using std::make_shared;
+using std::shared_ptr;
+using std::string;
 using namespace fspp::fuse;
+using cpputils::set_thread_name;
 
-#define FUSE_OBJ ((Fuse *) fuse_get_context()->private_data)
+namespace {
+bool is_valid_fspp_path(const bf::path& path) {
+  // TODO In boost 1.63, we can use path.generic() or path.generic_path() instead of path.generic_string()
+  return path.has_root_directory()                     // must be absolute path
+         && !path.has_root_name()                      // on Windows, it shouldn't have a device specifier (i.e. no "C:")
+         && (path.string() == path.generic_string());  // must use portable '/' as directory separator
+}
+
+class ThreadNameForDebugging final {
+public:
+  ThreadNameForDebugging(const string& threadName) {
+    std::string name = "fspp_" + threadName;
+    set_thread_name(name.c_str());
+  }
+
+  ~ThreadNameForDebugging() {
+    set_thread_name("fspp_idle");
+  }
+};
+}
+
+#define FUSE_OBJ (static_cast<Fuse *>(fuse_get_context()->private_data))
 
 // Remove the following line, if you don't want to output each fuse operation on the console
 //#define FSPP_LOG 1
 
 namespace {
-int fusepp_getattr(const char *path, struct stat *stbuf) {
+int fusepp_getattr(const char *path, fspp::fuse::STAT *stbuf) {
   int rs = FUSE_OBJ->getattr(bf::path(path), stbuf);
   return rs;
 }
 
-int fusepp_fgetattr(const char *path, struct stat *stbuf, fuse_file_info *fileinfo) {
+int fusepp_fgetattr(const char *path, fspp::fuse::STAT *stbuf, fuse_file_info *fileinfo) {
   return FUSE_OBJ->fgetattr(bf::path(path), stbuf, fileinfo);
 }
 
@@ -35,11 +69,11 @@ int fusepp_readlink(const char *path, char *buf, size_t size) {
   return FUSE_OBJ->readlink(bf::path(path), buf, size);
 }
 
-int fusepp_mknod(const char *path, mode_t mode, dev_t rdev) {
+int fusepp_mknod(const char *path, ::mode_t mode, dev_t rdev) {
   return FUSE_OBJ->mknod(bf::path(path), mode, rdev);
 }
 
-int fusepp_mkdir(const char *path, mode_t mode) {
+int fusepp_mkdir(const char *path, ::mode_t mode) {
   return FUSE_OBJ->mkdir(bf::path(path), mode);
 }
 
@@ -63,24 +97,24 @@ int fusepp_link(const char *from, const char *to) {
   return FUSE_OBJ->link(bf::path(from), bf::path(to));
 }
 
-int fusepp_chmod(const char *path, mode_t mode) {
+int fusepp_chmod(const char *path, ::mode_t mode) {
   return FUSE_OBJ->chmod(bf::path(path), mode);
 }
 
-int fusepp_chown(const char *path, uid_t uid, gid_t gid) {
+int fusepp_chown(const char *path, ::uid_t uid, ::gid_t gid) {
   return FUSE_OBJ->chown(bf::path(path), uid, gid);
 }
 
-int fusepp_truncate(const char *path, off_t size) {
+int fusepp_truncate(const char *path, int64_t size) {
   return FUSE_OBJ->truncate(bf::path(path), size);
 }
 
-int fusepp_ftruncate(const char *path, off_t size, fuse_file_info *fileinfo) {
+int fusepp_ftruncate(const char *path, int64_t size, fuse_file_info *fileinfo) {
   return FUSE_OBJ->ftruncate(bf::path(path), size, fileinfo);
 }
 
-int fusepp_utimens(const char *path, const timespec times[2]) {
-  return FUSE_OBJ->utimens(bf::path(path), times);
+int fusepp_utimens(const char *path, const timespec times[2]) {  // NOLINT(cppcoreguidelines-avoid-c-arrays)
+  return FUSE_OBJ->utimens(bf::path(path), {times[0], times[1]});
 }
 
 int fusepp_open(const char *path, fuse_file_info *fileinfo) {
@@ -91,11 +125,11 @@ int fusepp_release(const char *path, fuse_file_info *fileinfo) {
   return FUSE_OBJ->release(bf::path(path), fileinfo);
 }
 
-int fusepp_read(const char *path, char *buf, size_t size, off_t offset, fuse_file_info *fileinfo) {
+int fusepp_read(const char *path, char *buf, size_t size, int64_t offset, fuse_file_info *fileinfo) {
   return FUSE_OBJ->read(bf::path(path), buf, size, offset, fileinfo);
 }
 
-int fusepp_write(const char *path, const char *buf, size_t size, off_t offset, fuse_file_info *fileinfo) {
+int fusepp_write(const char *path, const char *buf, size_t size, int64_t offset, fuse_file_info *fileinfo) {
   return FUSE_OBJ->write(bf::path(path), buf, size, offset, fileinfo);
 }
 
@@ -120,7 +154,7 @@ int fusepp_opendir(const char *path, fuse_file_info *fileinfo) {
   return FUSE_OBJ->opendir(bf::path(path), fileinfo);
 }
 
-int fusepp_readdir(const char *path, void *buf, fuse_fill_dir_t filler, off_t offset, fuse_file_info *fileinfo) {
+int fusepp_readdir(const char *path, void *buf, fuse_fill_dir_t filler, int64_t offset, fuse_file_info *fileinfo) {
   return FUSE_OBJ->readdir(bf::path(path), buf, filler, offset, fileinfo);
 }
 
@@ -149,7 +183,7 @@ int fusepp_access(const char *path, int mask) {
   return FUSE_OBJ->access(bf::path(path), mask);
 }
 
-int fusepp_create(const char *path, mode_t mode, fuse_file_info *fileinfo) {
+int fusepp_create(const char *path, ::mode_t mode, fuse_file_info *fileinfo) {
   return FUSE_OBJ->create(bf::path(path), mode, fileinfo);
 }
 
@@ -157,10 +191,10 @@ int fusepp_create(const char *path, mode_t mode, fuse_file_info *fileinfo) {
 int fusepp_bmap(const char*, size_t blocksize, uint64_t *idx)
 int fusepp_ioctl(const char*, int cmd, void *arg, fuse_file_info*, unsigned int flags, void *data)
 int fusepp_poll(const char*, fuse_file_info*, fuse_pollhandle *ph, unsigned *reventsp)
-int fusepp_write_buf(const char*, fuse_bufvec *buf, off_t off, fuse_file_info*)
-int fusepp_read_buf(const chas*, struct fuse_bufvec **bufp, size_t size, off_T off, fuse_file_info*)
+int fusepp_write_buf(const char*, fuse_bufvec *buf, int64_t off, fuse_file_info*)
+int fusepp_read_buf(const chas*, struct fuse_bufvec **bufp, size_t size, int64_t off, fuse_file_info*)
 int fusepp_flock(const char*, fuse_file_info*, int op)
-int fusepp_fallocate(const char*, int, off_t, off_t, fuse_file_info*)*/
+int fusepp_fallocate(const char*, int, int64_t, int64_t, fuse_file_info*)*/
 
 fuse_operations *operations() {
   static std::unique_ptr<fuse_operations> singleton(nullptr);
@@ -211,44 +245,62 @@ fuse_operations *operations() {
 
 Fuse::~Fuse() {
   for(char *arg : _argv) {
-    delete arg;
+    delete[] arg;
     arg = nullptr;
   }
   _argv.clear();
 }
 
-Fuse::Fuse(Filesystem *fs, const std::string &fstype, const boost::optional<std::string> &fsname)
-  :_fs(fs), _mountdir(), _running(false), _fstype(fstype), _fsname(fsname) {
+Fuse::Fuse(std::function<shared_ptr<Filesystem> (Fuse *fuse)> init, std::function<void()> onMounted, std::string fstype, boost::optional<std::string> fsname)
+  :_init(std::move(init)), _onMounted(std::move(onMounted)), _fs(make_shared<InvalidFilesystem>()), _mountdir(), _running(false), _fstype(std::move(fstype)), _fsname(std::move(fsname)) {
+  ASSERT(static_cast<bool>(_init), "Invalid init given");
+  ASSERT(static_cast<bool>(_onMounted), "Invalid onMounted given");
 }
 
 void Fuse::_logException(const std::exception &e) {
-  LOG(ERROR, "Exception thrown: ", e.what());
+  LOG(ERR, "Exception thrown: {}", e.what());
 }
 
 void Fuse::_logUnknownException() {
-  LOG(ERROR, "Unknown exception thrown");
+  LOG(ERR, "Unknown exception thrown");
 }
 
 void Fuse::run(const bf::path &mountdir, const vector<string> &fuseOptions) {
+#if defined(__GLIBC__)|| defined(__APPLE__) || defined(_MSC_VER)
+  // Avoid encoding errors for non-utf8 characters, see https://github.com/cryfs/cryfs/issues/247
+  // this is ifdef'd out for non-glibc linux, because musl doesn't handle this correctly.
+  bf::path::imbue(std::locale(std::locale(), new std::codecvt_utf8_utf16<wchar_t>()));
+#endif
+
   _mountdir = mountdir;
 
   ASSERT(_argv.size() == 0, "Filesystem already started");
 
   _argv = _build_argv(mountdir, fuseOptions);
 
-  fuse_main(_argv.size(), _argv.data(), operations(), (void*)this);
+  fuse_main(_argv.size(), _argv.data(), operations(), this);
 }
 
 vector<char *> Fuse::_build_argv(const bf::path &mountdir, const vector<string> &fuseOptions) {
   vector<char *> argv;
   argv.reserve(6 + fuseOptions.size()); // fuseOptions + executable name + mountdir + 2x fuse options (subtype, fsname), each taking 2 entries ("-o", "key=value").
   argv.push_back(_create_c_string(_fstype)); // The first argument (executable name) is the file system type
-  argv.push_back(_create_c_string(mountdir.native())); // The second argument is the mountdir
+  argv.push_back(_create_c_string(mountdir.string())); // The second argument is the mountdir
   for (const string &option : fuseOptions) {
     argv.push_back(_create_c_string(option));
   }
   _add_fuse_option_if_not_exists(&argv, "subtype", _fstype);
   _add_fuse_option_if_not_exists(&argv, "fsname", _fsname.get_value_or(_fstype));
+#ifdef __APPLE__
+  // Make volume name default to mountdir on macOS
+  _add_fuse_option_if_not_exists(&argv, "volname", mountdir.filename().string());
+#endif
+  // TODO Also set read/write size for osxfuse. The options there are called differently.
+  // large_read not necessary because reads are large anyhow. This option is only important for 2.4.
+  //argv.push_back(_create_c_string("-o"));
+  //argv.push_back(_create_c_string("large_read"));
+  argv.push_back(_create_c_string("-o"));
+  argv.push_back(_create_c_string("big_writes"));
   return argv;
 }
 
@@ -283,28 +335,48 @@ bool Fuse::running() const {
 }
 
 void Fuse::stop() {
+  unmount(_mountdir, false);
+}
+
+void Fuse::unmount(const bf::path& mountdir, bool force) {
   //TODO Find better way to unmount (i.e. don't use external fusermount). Unmounting by kill(getpid(), SIGINT) worked, but left the mount directory transport endpoint as not connected.
-#ifdef __APPLE__
-  int ret = system(("umount " + _mountdir.native()).c_str());
+#if defined(__APPLE__)
+  UNUSED(force);
+  int returncode = cpputils::Subprocess::call(std::string("umount ") + mountdir.string()).exitcode;
+#elif defined(_MSC_VER)
+  UNUSED(force);
+  std::wstring mountdir_ = std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>>().from_bytes(mountdir.string());
+  BOOL success = DokanRemoveMountPoint(mountdir_.c_str());
+  int returncode = success ? 0 : -1;
 #else
-  int ret = system(("fusermount -z -u " + _mountdir.native()).c_str()); // "-z" takes care that if the filesystem can't be unmounted right now because something is opened, it will be unmounted as soon as it can be.
+  std::string command = force ? "fusermount -u" : "fusermount -z -u";  // "-z" takes care that if the filesystem can't be unmounted right now because something is opened, it will be unmounted as soon as it can be.
+  int returncode = cpputils::Subprocess::call(
+	  command + " " + mountdir.string()).exitcode;
 #endif
-  if (ret != 0) {
-    LOG(ERROR, "Could not unmount filesystem");
+  if (returncode != 0) {
+    throw std::runtime_error("Could not unmount filesystem");
   }
 }
 
-int Fuse::getattr(const bf::path &path, struct stat *stbuf) {
+int Fuse::getattr(const bf::path &path, fspp::fuse::STAT *stbuf) {
+  ThreadNameForDebugging _threadName("getattr");
 #ifdef FSPP_LOG
   LOG(DEBUG, "getattr({}, _, _)", path);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
     _fs->lstat(path, stbuf);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "getattr({}, _, _): success", path);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::getattr: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::getattr: {}", e.what());
     return -EIO;
-  } catch(fspp::fuse::FuseErrnoException &e) {
+  } catch(const fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "getattr({}, _, _): failed with errno {}", path, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -315,9 +387,10 @@ int Fuse::getattr(const bf::path &path, struct stat *stbuf) {
   }
 }
 
-int Fuse::fgetattr(const bf::path &path, struct stat *stbuf, fuse_file_info *fileinfo) {
+int Fuse::fgetattr(const bf::path &path, fspp::fuse::STAT *stbuf, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("fgetattr");
 #ifdef FSPP_LOG
-  LOG(DEBUG, "fgetattr({}, _, _)\n", path);
+  LOG(DEBUG, "fgetattr({}, _, _)", path);
 #endif
 
   // On FreeBSD, trying to do anything with the mountpoint ends up
@@ -325,17 +398,28 @@ int Fuse::fgetattr(const bf::path &path, struct stat *stbuf, fuse_file_info *fil
   // special case of a path of "/", I need to do a getattr on the
   // underlying base directory instead of doing the fgetattr().
   // TODO Check if necessary
-  if (path.native() == "/") {
-    return getattr(path, stbuf);
+  if (path.string() == "/") {
+    int result = getattr(path, stbuf);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "fgetattr({}, _, _): success", path);
+#endif
+    return result;
   }
 
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
     _fs->fstat(fileinfo->fh, stbuf);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "fgetattr({}, _, _): success", path);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::fgetattr: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::fgetattr: {}", e.what());
     return -EIO;
-  } catch(fspp::fuse::FuseErrnoException &e) {
+  } catch(const fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+      LOG(ERR, "fgetattr({}, _, _): error", path);
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -347,16 +431,24 @@ int Fuse::fgetattr(const bf::path &path, struct stat *stbuf, fuse_file_info *fil
 }
 
 int Fuse::readlink(const bf::path &path, char *buf, size_t size) {
+  ThreadNameForDebugging _threadName("readlink");
 #ifdef FSPP_LOG
   LOG(DEBUG, "readlink({}, _, {})", path, size);
 #endif
   try {
-    _fs->readSymlink(path, buf, size);
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
+    _fs->readSymlink(path, buf, fspp::num_bytes_t(size));
+#ifdef FSPP_LOG
+    LOG(DEBUG, "readlink({}, _, {}): success", path, size);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::readlink: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::readlink: {}", e.what());
     return -EIO;
   } catch (fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "readlink({}, _, {}): failed with errno {}", path, size, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -367,26 +459,43 @@ int Fuse::readlink(const bf::path &path, char *buf, size_t size) {
   }
 }
 
-int Fuse::mknod(const bf::path &path, mode_t mode, dev_t rdev) {
+int Fuse::mknod(const bf::path &path, ::mode_t mode, dev_t rdev) {
   UNUSED(rdev);
   UNUSED(mode);
   UNUSED(path);
+  ThreadNameForDebugging _threadName("mknod");
   LOG(WARN, "Called non-implemented mknod({}, {}, _)", path, mode);
   return ENOSYS;
 }
 
-int Fuse::mkdir(const bf::path &path, mode_t mode) {
+int Fuse::mkdir(const bf::path &path, ::mode_t mode) {
+  ThreadNameForDebugging _threadName("mkdir");
 #ifdef FSPP_LOG
   LOG(DEBUG, "mkdir({}, {})", path, mode);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
+	// DokanY seems to call mkdir("/"). Ignore that
+	if ("/" == path) {
+#ifdef FSPP_LOG
+        LOG(DEBUG, "mkdir({}, {}): ignored", path, mode);
+#endif
+		return 0;
+	}
+
     auto context = fuse_get_context();
     _fs->mkdir(path, mode, context->uid, context->gid);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "mkdir({}, {}): success", path, mode);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::mkdir: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::mkdir: {}", e.what());
     return -EIO;
-  } catch(fspp::fuse::FuseErrnoException &e) {
+  } catch(const fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "mkdir({}, {}): failed with errno {}", path, mode, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -398,16 +507,24 @@ int Fuse::mkdir(const bf::path &path, mode_t mode) {
 }
 
 int Fuse::unlink(const bf::path &path) {
+  ThreadNameForDebugging _threadName("unlink");
 #ifdef FSPP_LOG
   LOG(DEBUG, "unlink({})", path);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
     _fs->unlink(path);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "unlink({}): success", path);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::unlink: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::unlink: {}", e.what());
     return -EIO;
-  } catch(fspp::fuse::FuseErrnoException &e) {
+  } catch(const fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "unlink({}): failed with errno {}", path, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -419,16 +536,24 @@ int Fuse::unlink(const bf::path &path) {
 }
 
 int Fuse::rmdir(const bf::path &path) {
+  ThreadNameForDebugging _threadName("rmdir");
 #ifdef FSPP_LOG
   LOG(DEBUG, "rmdir({})", path);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
     _fs->rmdir(path);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "rmdir({}): success", path);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::rmdir: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::rmdir: {}", e.what());
     return -EIO;
-  } catch(fspp::fuse::FuseErrnoException &e) {
+  } catch(const fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "rmdir({}): failed with errno {}", path, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -439,18 +564,26 @@ int Fuse::rmdir(const bf::path &path) {
   }
 }
 
-int Fuse::symlink(const bf::path &from, const bf::path &to) {
+int Fuse::symlink(const bf::path &to, const bf::path &from) {
+  ThreadNameForDebugging _threadName("symlink");
 #ifdef FSPP_LOG
-  LOG(DEBUG, "symlink({}, {})", from, to);
+  LOG(DEBUG, "symlink({}, {})", to, from);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(from), "has to be an absolute path");
 	auto context = fuse_get_context();
-    _fs->createSymlink(from, to, context->uid, context->gid);
+    _fs->createSymlink(to, from, context->uid, context->gid);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "symlink({}, {}): success", to, from);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::symlink: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::symlink: {}", e.what());
     return -EIO;
-  } catch(fspp::fuse::FuseErrnoException &e) {
+  } catch(const fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "symlink({}, {}): failed with errno {}", to, from, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -462,18 +595,25 @@ int Fuse::symlink(const bf::path &from, const bf::path &to) {
 }
 
 int Fuse::rename(const bf::path &from, const bf::path &to) {
+  ThreadNameForDebugging _threadName("rename");
 #ifdef FSPP_LOG
   LOG(DEBUG, "rename({}, {})", from, to);
 #endif
   try {
-    ASSERT(from.is_absolute(), "from has to be an absolute path");
-    ASSERT(to.is_absolute(), "rename target has to be an absolute path. If this assert throws, we have to add code here that makes the path absolute.");
+    ASSERT(is_valid_fspp_path(from), "from has to be an absolute path");
+    ASSERT(is_valid_fspp_path(to), "rename target has to be an absolute path. If this assert throws, we have to add code here that makes the path absolute.");
     _fs->rename(from, to);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "rename({}, {}): success", from, to);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::rename: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::rename: {}", e.what());
     return -EIO;
-  } catch(fspp::fuse::FuseErrnoException &e) {
+  } catch(const fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "rename({}, {}): failed with errno {}", from, to, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -486,25 +626,34 @@ int Fuse::rename(const bf::path &from, const bf::path &to) {
 
 //TODO
 int Fuse::link(const bf::path &from, const bf::path &to) {
+  ThreadNameForDebugging _threadName("link");
   LOG(WARN, "NOT IMPLEMENTED: link({}, {})", from, to);
   //auto real_from = _impl->RootDir() / from;
   //auto real_to = _impl->RootDir() / to;
-  //int retstat = ::link(real_from.c_str(), real_to.c_str());
+  //int retstat = ::link(real_from.string().c_str(), real_to.string().c_str());
   //return errcode_map(retstat);
   return ENOSYS;
 }
 
-int Fuse::chmod(const bf::path &path, mode_t mode) {
+int Fuse::chmod(const bf::path &path, ::mode_t mode) {
+  ThreadNameForDebugging _threadName("chmod");
 #ifdef FSPP_LOG
   LOG(DEBUG, "chmod({}, {})", path, mode);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
 	_fs->chmod(path, mode);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "chmod({}, {}): success", path, mode);
+#endif
 	return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::chmod: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::chmod: {}", e.what());
     return -EIO;
   } catch (fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "chmod({}, {}): failed with errno {}", path, mode, e.getErrno());
+#endif
 	return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -515,17 +664,25 @@ int Fuse::chmod(const bf::path &path, mode_t mode) {
   }
 }
 
-int Fuse::chown(const bf::path &path, uid_t uid, gid_t gid) {
+int Fuse::chown(const bf::path &path, ::uid_t uid, ::gid_t gid) {
+  ThreadNameForDebugging _threadName("chown");
 #ifdef FSPP_LOG
   LOG(DEBUG, "chown({}, {}, {})", path, uid, gid);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
 	_fs->chown(path, uid, gid);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "chown({}, {}, {}): success", path, uid, gid);
+#endif
 	return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::chown: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::chown: {}", e.what());
     return -EIO;
   } catch (fspp::fuse::FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "chown({}, {}, {}): failed with errno {}", path, uid, gid, e.getErrno());
+#endif
 	return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -536,17 +693,25 @@ int Fuse::chown(const bf::path &path, uid_t uid, gid_t gid) {
   }
 }
 
-int Fuse::truncate(const bf::path &path, off_t size) {
+int Fuse::truncate(const bf::path &path, int64_t size) {
+  ThreadNameForDebugging _threadName("truncate");
 #ifdef FSPP_LOG
   LOG(DEBUG, "truncate({}, {})", path, size);
 #endif
   try {
-    _fs->truncate(path, size);
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
+    _fs->truncate(path, fspp::num_bytes_t(size));
+#ifdef FSPP_LOG
+    LOG(DEBUG, "truncate({}, {}): success", path, size);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::truncate: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::truncate: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "truncate({}, {}): failed with errno {}", path, size, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -557,18 +722,25 @@ int Fuse::truncate(const bf::path &path, off_t size) {
   }
 }
 
-int Fuse::ftruncate(const bf::path &path, off_t size, fuse_file_info *fileinfo) {
+int Fuse::ftruncate(const bf::path &path, int64_t size, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("ftruncate");
 #ifdef FSPP_LOG
   LOG(DEBUG, "ftruncate({}, {})", path, size);
 #endif
   UNUSED(path);
   try {
-    _fs->ftruncate(fileinfo->fh, size);
+    _fs->ftruncate(fileinfo->fh, fspp::num_bytes_t(size));
+#ifdef FSPP_LOG
+    LOG(DEBUG, "ftruncate({}, {}): success", path, size);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::ftruncate: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::ftruncate: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "ftruncate({}, {}): failed with errno {}", path, size, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -579,17 +751,25 @@ int Fuse::ftruncate(const bf::path &path, off_t size, fuse_file_info *fileinfo)
   }
 }
 
-int Fuse::utimens(const bf::path &path, const timespec times[2]) {
+int Fuse::utimens(const bf::path &path, const std::array<timespec, 2> times) {
+  ThreadNameForDebugging _threadName("utimens");
 #ifdef FSPP_LOG
   LOG(DEBUG, "utimens({}, _)", path);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
     _fs->utimens(path, times[0], times[1]);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "utimens({}, _): success", path);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::utimens: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::utimens: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "utimens({}, _): failed with errno {}", path, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -601,16 +781,24 @@ int Fuse::utimens(const bf::path &path, const timespec times[2]) {
 }
 
 int Fuse::open(const bf::path &path, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("open");
 #ifdef FSPP_LOG
   LOG(DEBUG, "open({}, _)", path);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
     fileinfo->fh = _fs->openFile(path, fileinfo->flags);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "open({}, _): success", path);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::open: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::open: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "open({}, _): failed with errno {}", path, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -622,17 +810,24 @@ int Fuse::open(const bf::path &path, fuse_file_info *fileinfo) {
 }
 
 int Fuse::release(const bf::path &path, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("release");
 #ifdef FSPP_LOG
   LOG(DEBUG, "release({}, _)", path);
 #endif
   UNUSED(path);
   try {
     _fs->closeFile(fileinfo->fh);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "release({}, _): success", path);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::release: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::release: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "release({}, _): failed with errno {}", path, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -643,17 +838,25 @@ int Fuse::release(const bf::path &path, fuse_file_info *fileinfo) {
   }
 }
 
-int Fuse::read(const bf::path &path, char *buf, size_t size, off_t offset, fuse_file_info *fileinfo) {
+int Fuse::read(const bf::path &path, char *buf, size_t size, int64_t offset, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("read");
 #ifdef FSPP_LOG
   LOG(DEBUG, "read({}, _, {}, {}, _)", path, size, offset);
 #endif
   UNUSED(path);
   try {
-    return _fs->read(fileinfo->fh, buf, size, offset);
+    int result = _fs->read(fileinfo->fh, buf, fspp::num_bytes_t(size), fspp::num_bytes_t(offset)).value();
+#ifdef FSPP_LOG
+    LOG(DEBUG, "read({}, _, {}, {}, _): success with {}", path, size, offset, result);
+#endif
+    return result;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::read: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::read: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "read({}, _, {}, {}, _): failed with errno {}", path, size, offset, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -664,18 +867,25 @@ int Fuse::read(const bf::path &path, char *buf, size_t size, off_t offset, fuse_
   }
 }
 
-int Fuse::write(const bf::path &path, const char *buf, size_t size, off_t offset, fuse_file_info *fileinfo) {
+int Fuse::write(const bf::path &path, const char *buf, size_t size, int64_t offset, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("write");
 #ifdef FSPP_LOG
-  LOG(DEBUG, "write({}, _, {}, {}, _)", path, size, offsset);
+  LOG(DEBUG, "write({}, _, {}, {}, _)", path, size, offset);
 #endif
   UNUSED(path);
   try {
-    _fs->write(fileinfo->fh, buf, size, offset);
+    _fs->write(fileinfo->fh, buf, fspp::num_bytes_t(size), fspp::num_bytes_t(offset));
+#ifdef FSPP_LOG
+    LOG(DEBUG, "write({}, _, {}, {}, _): success", path, size, offset);
+#endif
     return size;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::write: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::write: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "write({}, _, {}, {}, _): failed with errno {}", path, size, offset, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -686,18 +896,26 @@ int Fuse::write(const bf::path &path, const char *buf, size_t size, off_t offset
   }
 }
 
-//TODO
-int Fuse::statfs(const bf::path &path, struct statvfs *fsstat) {
+int Fuse::statfs(const bf::path &path, struct ::statvfs *fsstat) {
+  ThreadNameForDebugging _threadName("statfs");
 #ifdef FSPP_LOG
   LOG(DEBUG, "statfs({}, _)", path);
 #endif
+  UNUSED(path);
   try {
-    _fs->statfs(path, fsstat);
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
+    _fs->statfs(fsstat);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "statfs({}, _): success", path);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::statfs: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::statfs: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "statfs({}, _): failed with errno {}", path, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -709,17 +927,24 @@ int Fuse::statfs(const bf::path &path, struct statvfs *fsstat) {
 }
 
 int Fuse::flush(const bf::path &path, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("flush");
 #ifdef FSPP_LOG
   LOG(WARN, "flush({}, _)", path);
 #endif
   UNUSED(path);
   try {
     _fs->flush(fileinfo->fh);
+#ifdef FSPP_LOG
+    LOG(WARN, "flush({}, _): success", path);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::flush: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::flush: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "flush({}, _): failed with errno {}", path, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -731,6 +956,7 @@ int Fuse::flush(const bf::path &path, fuse_file_info *fileinfo) {
 }
 
 int Fuse::fsync(const bf::path &path, int datasync, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("fsync");
 #ifdef FSPP_LOG
   LOG(DEBUG, "fsync({}, {}, _)", path, datasync);
 #endif
@@ -741,11 +967,17 @@ int Fuse::fsync(const bf::path &path, int datasync, fuse_file_info *fileinfo) {
     } else {
       _fs->fsync(fileinfo->fh);
     }
+#ifdef FSPP_LOG
+  LOG(DEBUG, "fsync({}, {}, _): success", path, datasync);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::fsync: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::fsync: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "fsync({}, {}, _): failed with errno {}", path, datasync, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -759,20 +991,23 @@ int Fuse::fsync(const bf::path &path, int datasync, fuse_file_info *fileinfo) {
 int Fuse::opendir(const bf::path &path, fuse_file_info *fileinfo) {
   UNUSED(path);
   UNUSED(fileinfo);
+  ThreadNameForDebugging _threadName("opendir");
   //LOG(DEBUG, "opendir({}, _)", path);
   //We don't need opendir, because readdir works directly on the path
   return 0;
 }
 
-int Fuse::readdir(const bf::path &path, void *buf, fuse_fill_dir_t filler, off_t offset, fuse_file_info *fileinfo) {
+int Fuse::readdir(const bf::path &path, void *buf, fuse_fill_dir_t filler, int64_t offset, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("readdir");
 #ifdef FSPP_LOG
-  LOG(DEBUG, "readdir({}, _, _, {}, _)", path, offest);
+  LOG(DEBUG, "readdir({}, _, _, {}, _)", path, offset);
 #endif
   UNUSED(fileinfo);
   UNUSED(offset);
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
     auto entries = _fs->readDir(path);
-    struct stat stbuf;
+    fspp::fuse::STAT stbuf{};
     for (const auto &entry : *entries) {
       //We could pass more file metadata to filler() in its third parameter,
       //but it doesn't help performance since fuse ignores everything in stbuf
@@ -788,14 +1023,23 @@ int Fuse::readdir(const bf::path &path, void *buf, fuse_fill_dir_t filler, off_t
         ASSERT(false, "Unknown entry type");
       }
       if (filler(buf, entry.name.c_str(), &stbuf, 0) != 0) {
+#ifdef FSPP_LOG
+        LOG(DEBUG, "readdir({}, _, _, {}, _): failure with ENOMEM", path, offset);
+#endif
         return -ENOMEM;
       }
     }
+#ifdef FSPP_LOG
+    LOG(DEBUG, "readdir({}, _, _, {}, _): success", path, offset);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::readdir: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::readdir: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "readdir({}, _, _, {}, _): failed with errno {}", path, offset, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -809,6 +1053,7 @@ int Fuse::readdir(const bf::path &path, void *buf, fuse_fill_dir_t filler, off_t
 int Fuse::releasedir(const bf::path &path, fuse_file_info *fileinfo) {
   UNUSED(path);
   UNUSED(fileinfo);
+  ThreadNameForDebugging _threadName("releasedir");
   //LOG(DEBUG, "releasedir({}, _)", path);
   //We don't need releasedir, because readdir works directly on the path
   return 0;
@@ -819,15 +1064,20 @@ int Fuse::fsyncdir(const bf::path &path, int datasync, fuse_file_info *fileinfo)
   UNUSED(fileinfo);
   UNUSED(datasync);
   UNUSED(path);
+  ThreadNameForDebugging _threadName("fsyncdir");
   //LOG(WARN, "Called non-implemented fsyncdir({}, {}, _)", path, datasync);
   return 0;
 }
 
 void Fuse::init(fuse_conn_info *conn) {
   UNUSED(conn);
+  ThreadNameForDebugging _threadName("init");
+  _fs = _init(this);
+
   LOG(INFO, "Filesystem started.");
 
   _running = true;
+  _onMounted();
 
 #ifdef FSPP_LOG
   cpputils::logging::setLevel(DEBUG);
@@ -835,21 +1085,32 @@ void Fuse::init(fuse_conn_info *conn) {
 }
 
 void Fuse::destroy() {
+  ThreadNameForDebugging _threadName("destroy");
+  _fs = make_shared<InvalidFilesystem>();
   LOG(INFO, "Filesystem stopped.");
   _running = false;
+  cpputils::logging::logger()->flush();
 }
 
 int Fuse::access(const bf::path &path, int mask) {
+  ThreadNameForDebugging _threadName("access");
 #ifdef FSPP_LOG
   LOG(DEBUG, "access({}, {})", path, mask);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
     _fs->access(path, mask);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "access({}, {}): success", path, mask);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::access: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::access: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "access({}, {}): failed with errno {}", path, mask, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
@@ -860,18 +1121,26 @@ int Fuse::access(const bf::path &path, int mask) {
   }
 }
 
-int Fuse::create(const bf::path &path, mode_t mode, fuse_file_info *fileinfo) {
+int Fuse::create(const bf::path &path, ::mode_t mode, fuse_file_info *fileinfo) {
+  ThreadNameForDebugging _threadName("create");
 #ifdef FSPP_LOG
   LOG(DEBUG, "create({}, {}, _)", path, mode);
 #endif
   try {
+    ASSERT(is_valid_fspp_path(path), "has to be an absolute path");
     auto context = fuse_get_context();
     fileinfo->fh = _fs->createAndOpenFile(path, mode, context->uid, context->gid);
+#ifdef FSPP_LOG
+    LOG(DEBUG, "create({}, {}, _): success", path, mode);
+#endif
     return 0;
   } catch(const cpputils::AssertFailed &e) {
-    LOG(ERROR, "AssertFailed in Fuse::create: {}", e.what());
+    LOG(ERR, "AssertFailed in Fuse::create: {}", e.what());
     return -EIO;
   } catch (FuseErrnoException &e) {
+#ifdef FSPP_LOG
+    LOG(WARN, "create({}, {}, _): failed with errno {}", path, mode, e.getErrno());
+#endif
     return -e.getErrno();
   } catch(const std::exception &e) {
     _logException(e);
