@@ -21,33 +21,33 @@ class OpenFile;
 
 class FilesystemImpl final: public fuse::Filesystem {
 public:
-  explicit FilesystemImpl(Device *device);
+  explicit FilesystemImpl(cpputils::unique_ref<Device> device);
 	virtual ~FilesystemImpl();
 
 	int openFile(const boost::filesystem::path &path, int flags) override;
 	void flush(int descriptor) override;
 	void closeFile(int descriptor) override;
-	void lstat(const boost::filesystem::path &path, struct ::stat *stbuf) override;
-	void fstat(int descriptor, struct ::stat *stbuf) override;
-	void chmod(const boost::filesystem::path &path, mode_t mode) override;
-	void chown(const boost::filesystem::path &path, uid_t uid, gid_t gid) override;
-	void truncate(const boost::filesystem::path &path, off_t size) override;
-	void ftruncate(int descriptor, off_t size) override;
-	size_t read(int descriptor, void *buf, size_t count, off_t offset) override;
-	void write(int descriptor, const void *buf, size_t count, off_t offset) override;
+	void lstat(const boost::filesystem::path &path, fspp::fuse::STAT *stbuf) override;
+	void fstat(int descriptor, fspp::fuse::STAT *stbuf) override;
+	void chmod(const boost::filesystem::path &path, ::mode_t mode) override;
+	void chown(const boost::filesystem::path &path, ::uid_t uid, ::gid_t gid) override;
+	void truncate(const boost::filesystem::path &path, fspp::num_bytes_t size) override;
+	void ftruncate(int descriptor, fspp::num_bytes_t size) override;
+	fspp::num_bytes_t read(int descriptor, void *buf, fspp::num_bytes_t count, fspp::num_bytes_t offset) override;
+	void write(int descriptor, const void *buf, fspp::num_bytes_t count, fspp::num_bytes_t offset) override;
 	void fsync(int descriptor) override;
 	void fdatasync(int descriptor) override;
 	void access(const boost::filesystem::path &path, int mask) override;
-	int createAndOpenFile(const boost::filesystem::path &path, mode_t mode, uid_t uid, gid_t gid) override;
-	void mkdir(const boost::filesystem::path &path, mode_t mode, uid_t uid, gid_t gid) override;
+	int createAndOpenFile(const boost::filesystem::path &path, ::mode_t mode, ::uid_t uid, ::gid_t gid) override;
+	void mkdir(const boost::filesystem::path &path, ::mode_t mode, ::uid_t uid, ::gid_t gid) override;
 	void rmdir(const boost::filesystem::path &path) override;
 	void unlink(const boost::filesystem::path &path) override;
 	void rename(const boost::filesystem::path &from, const boost::filesystem::path &to) override;
 	cpputils::unique_ref<std::vector<Dir::Entry>> readDir(const boost::filesystem::path &path) override;
 	void utimens(const boost::filesystem::path &path, timespec lastAccessTime, timespec lastModificationTime) override;
-	void statfs(const boost::filesystem::path &path, struct statvfs *fsstat) override;
-    void createSymlink(const boost::filesystem::path &to, const boost::filesystem::path &from, uid_t uid, gid_t gid) override;
-    void readSymlink(const boost::filesystem::path &path, char *buf, size_t size) override;
+	void statfs(struct ::statvfs *fsstat) override;
+    void createSymlink(const boost::filesystem::path &to, const boost::filesystem::path &from, ::uid_t uid, ::gid_t gid) override;
+    void readSymlink(const boost::filesystem::path &path, char *buf, fspp::num_bytes_t size) override;
 
 private:
 	cpputils::unique_ref<File> LoadFile(const boost::filesystem::path &path);
@@ -93,7 +93,7 @@ private:
     std::atomic<uint64_t> _readSymlinkNanosec_withoutLoading;
 #endif
 
-	Device *_device;
+	cpputils::unique_ref<Device> _device;
 	FuseOpenFileList _open_files;
 
   DISALLOW_COPY_AND_ASSIGN(FilesystemImpl);
