@@ -1,6 +1,8 @@
 #include <gtest/gtest.h>
 #include <cryfs/config/CryConfigLoader.h>
+#include <cryfs/config/CryPresetPasswordBasedKeyProvider.h>
 #include "../testutils/MockConsole.h"
+#include "../testutils/TestWithFakeHomeDirectory.h"
 #include <cpp-utils/tempfile/TempFile.h>
 #include <cpp-utils/random/Random.h>
 #include <cpp-utils/crypto/symmetric/ciphers.h>
@@ -10,20 +12,24 @@
 #include <gitversion/parser.h>
 #include <gitversion/VersionCompare.h>
 
-using cpputils::unique_ref;
-using cpputils::make_unique_ref;
 using cpputils::TempFile;
 using cpputils::SCrypt;
 using cpputils::DataFixture;
+using cpputils::Data;
 using cpputils::NoninteractiveConsole;
+using cpputils::make_unique_ref;
+using cpputils::Console;
+using cpputils::unique_ref;
+using cryfs::CryPresetPasswordBasedKeyProvider;
 using boost::optional;
 using boost::none;
 using std::string;
 using std::ostream;
+using std::shared_ptr;
 using std::make_shared;
 using ::testing::Return;
-using ::testing::_;
 using ::testing::HasSubstr;
+using ::testing::_;
 
 using namespace cryfs;
 
@@ -33,72 +39,105 @@ namespace boost {
         return stream << "CryConfigFile()";
     }
 }
+namespace cryfs {
+  inline ostream &operator<<(ostream &stream, const CryConfigLoader::ConfigLoadResult &) {
+    return stream << "ConfigLoadResult()";
+  }
+}
 #include <boost/optional/optional_io.hpp>
 
-class CryConfigLoaderTest: public ::testing::Test, public TestWithMockConsole {
+class FakeRandomGenerator final : public cpputils::RandomGenerator {
 public:
-    CryConfigLoaderTest(): file(false) {
+  FakeRandomGenerator(Data output)
+      : _output(std::move(output)) {}
+
+  void _get(void *target, size_t bytes) override {
+    ASSERT_EQ(_output.size(), bytes);
+    std::memcpy(target, _output.data(), bytes);
+  }
+
+private:
+  Data _output;
+};
+
+class CryConfigLoaderTest: public ::testing::Test, public TestWithMockConsole, TestWithFakeHomeDirectory {
+public:
+    unique_ref<CryKeyProvider> keyProvider(const string& password) {
+      return make_unique_ref<CryPresetPasswordBasedKeyProvider>(password, make_unique_ref<SCrypt>(SCrypt::TestSettings));
+    }
+
+    CryConfigLoaderTest(): file(false), tempLocalStateDir(), localStateDir(tempLocalStateDir.path()) {
         console = mockConsole();
     }
 
     CryConfigLoader loader(const string &password, bool noninteractive, const optional<string> &cipher = none) {
-        auto askPassword = [password] { return password;};
-        if(noninteractive) {
-            return CryConfigLoader(make_shared<NoninteractiveConsole>(console), cpputils::Random::PseudoRandom(), SCrypt::TestSettings, askPassword,
-                                   askPassword, cipher, none);
-        } else {
-            return CryConfigLoader(console, cpputils::Random::PseudoRandom(), SCrypt::TestSettings, askPassword,
-                                   askPassword, cipher, none);
-        }
+        auto _console = noninteractive ? shared_ptr<Console>(make_shared<NoninteractiveConsole>(console)) : shared_ptr<Console>(console);
+        return CryConfigLoader(_console, cpputils::Random::PseudoRandom(), keyProvider(password), localStateDir, cipher, none, none);
     }
 
     CryConfigFile Create(const string &password = "mypassword", const optional<string> &cipher = none, bool noninteractive = false) {
         EXPECT_FALSE(file.exists());
-        return loader(password, noninteractive, cipher).loadOrCreate(file.path(), false).value();
+        return loader(password, noninteractive, cipher).loadOrCreate(file.path(), false, false).value().configFile;
     }
 
     optional<CryConfigFile> Load(const string &password = "mypassword", const optional<string> &cipher = none, bool noninteractive = false, bool allowFilesystemUpgrade = false) {
         EXPECT_TRUE(file.exists());
-        return loader(password, noninteractive, cipher).loadOrCreate(file.path(), allowFilesystemUpgrade);
+        auto loadResult = loader(password, noninteractive, cipher).loadOrCreate(file.path(), allowFilesystemUpgrade, false);
+        if (loadResult == none) {
+            return none;
+        }
+        return std::move(loadResult->configFile);
     }
 
     void CreateWithRootBlob(const string &rootBlob, const string &password = "mypassword") {
-        auto cfg = loader(password, false).loadOrCreate(file.path(), false).value();
+        auto cfg = loader(password, false).loadOrCreate(file.path(), false, false).value().configFile;
         cfg.config()->SetRootBlob(rootBlob);
         cfg.save();
     }
 
     void CreateWithCipher(const string &cipher, const string &password = "mypassword") {
-        auto cfg = loader(password, false).loadOrCreate(file.path(), false).value();
+        auto cfg = loader(password, false).loadOrCreate(file.path(), false, false).value().configFile;
         cfg.config()->SetCipher(cipher);
         cfg.save();
     }
 
     void CreateWithEncryptionKey(const string &encKey, const string &password = "mypassword") {
-        auto cfg = loader(password, false).loadOrCreate(file.path(), false).value();
+        FakeRandomGenerator generator(Data::FromString(encKey));
+        auto loader = CryConfigLoader(console, generator, keyProvider(password), localStateDir, none, none, none);
+        ASSERT_NE(boost::none, loader.loadOrCreate(file.path(), false, false));
+    }
+
+    void ChangeEncryptionKey(const string &encKey, const string& password = "mypassword") {
+        auto cfg = CryConfigFile::load(file.path(), keyProvider(password).get()).value();
         cfg.config()->SetEncryptionKey(encKey);
         cfg.save();
     }
 
-    void CreateWithVersion(const string &version, const string &password = "mypassword") {
-        auto cfg = loader(password, false).loadOrCreate(file.path(), false).value();
-        cfg.config()->SetVersion(version);
+    void CreateWithVersion(const string &version, const string& formatVersion, const string &password = "mypassword") {
+        auto cfg = loader(password, false).loadOrCreate(file.path(), false, false).value().configFile;
+        cfg.config()->SetVersion(formatVersion);
         cfg.config()->SetLastOpenedWithVersion(version);
         cfg.config()->SetCreatedWithVersion(version);
         cfg.save();
     }
-
+  
     void CreateWithFilesystemID(const CryConfig::FilesystemID &filesystemId, const string &password = "mypassword") {
-        auto cfg = loader(password, false).loadOrCreate(file.path(), false).value();
+        auto cfg = loader(password, false).loadOrCreate(file.path(), false, false).value().configFile;
         cfg.config()->SetFilesystemId(filesystemId);
         cfg.save();
     }
 
+    void ChangeFilesystemID(const CryConfig::FilesystemID &filesystemId, const string& password = "mypassword") {
+      auto cfg = CryConfigFile::load(file.path(), keyProvider(password).get()).value();
+      cfg.config()->SetFilesystemId(filesystemId);
+      cfg.save();
+    }
+
     string olderVersion() {
         auto versionInfo = gitversion::Parser::parse(CryConfig::FilesystemFormatVersion);
         string olderVersion;
         if (std::stol(versionInfo.minorVersion) > 0) {
-            olderVersion = versionInfo.majorVersion + "." + std::to_string(std::stol(versionInfo.minorVersion) - 1);
+            olderVersion = versionInfo.majorVersion + "." + std::to_string(std::stol(versionInfo.minorVersion) - 1) + ".9";
         } else {
             olderVersion = std::to_string(std::stol(versionInfo.majorVersion) - 1) + "." + versionInfo.minorVersion;
         }
@@ -107,13 +146,16 @@ public:
     }
 
     string newerVersion() {
-        string newerVersion = gitversion::MajorVersion()+"."+std::to_string(std::stol(gitversion::MinorVersion())+1);
-        assert(gitversion::VersionCompare::isOlderThan(CryConfig::FilesystemFormatVersion, newerVersion));
+        string newerVersion = gitversion::MajorVersion()+"."+std::to_string(std::stol(gitversion::MinorVersion())+2);
+        EXPECT_TRUE(gitversion::VersionCompare::isOlderThan(CryConfig::FilesystemFormatVersion, newerVersion))
+            << "Format Version " << CryConfig::FilesystemFormatVersion << " should be older than Git Version " << newerVersion;
         return newerVersion;
     }
 
     std::shared_ptr<MockConsole> console;
     TempFile file;
+    cpputils::TempDir tempLocalStateDir;
+    LocalStateDir localStateDir;
 };
 
 TEST_F(CryConfigLoaderTest, CreatesNewIfNotExisting) {
@@ -175,9 +217,18 @@ TEST_F(CryConfigLoaderTest, RootBlob_Create) {
 }
 
 TEST_F(CryConfigLoaderTest, EncryptionKey_Load) {
-    CreateWithEncryptionKey("encryptionkey");
+    CreateWithEncryptionKey("3B4682CF22F3CA199E385729B9F3CA19D325229E385729B9443CA19D325229E3");
     auto loaded = Load().value();
-    EXPECT_EQ("encryptionkey", loaded.config()->EncryptionKey());
+    EXPECT_EQ("3B4682CF22F3CA199E385729B9F3CA19D325229E385729B9443CA19D325229E3", loaded.config()->EncryptionKey());
+}
+
+TEST_F(CryConfigLoaderTest, EncryptionKey_Load_whenKeyChanged_thenFails) {
+  CreateWithEncryptionKey("3B4682CF22F3CA199E385729B9F3CA19D325229E385729B9443CA19D325229E3");
+  ChangeEncryptionKey("3B4682CF22F3CA199E385729B9F3CA19D325229E385729B9443CA19D325229E4");
+  EXPECT_THROW(
+      Load(),
+      std::runtime_error
+  );
 }
 
 TEST_F(CryConfigLoaderTest, EncryptionKey_Create) {
@@ -199,20 +250,20 @@ TEST_F(CryConfigLoaderTest, Cipher_Create) {
 }
 
 TEST_F(CryConfigLoaderTest, Version_Load) {
-    CreateWithVersion("0.9.2");
+    CreateWithVersion("0.9.4", "0.9.4");
     auto loaded = Load().value();
     EXPECT_EQ(CryConfig::FilesystemFormatVersion, loaded.config()->Version());
     EXPECT_EQ(gitversion::VersionString(), loaded.config()->LastOpenedWithVersion());
-    EXPECT_EQ("0.9.2", loaded.config()->CreatedWithVersion());
+    EXPECT_EQ("0.9.4", loaded.config()->CreatedWithVersion());
 }
 
 TEST_F(CryConfigLoaderTest, Version_Load_IsStoredAndNotOnlyOverwrittenInMemoryOnLoad) {
-    CreateWithVersion("0.9.2", "mypassword");
+    CreateWithVersion("0.9.4", "0.9.4", "mypassword");
     Load().value();
-    auto configFile = CryConfigFile::load(file.path(), "mypassword").value();
+    auto configFile = CryConfigFile::load(file.path(), keyProvider("mypassword").get()).value();
     EXPECT_EQ(CryConfig::FilesystemFormatVersion, configFile.config()->Version());
     EXPECT_EQ(gitversion::VersionString(), configFile.config()->LastOpenedWithVersion());
-    EXPECT_EQ("0.9.2", configFile.config()->CreatedWithVersion());
+    EXPECT_EQ("0.9.4", configFile.config()->CreatedWithVersion());
 }
 
 TEST_F(CryConfigLoaderTest, Version_Create) {
@@ -238,7 +289,7 @@ TEST_F(CryConfigLoaderTest, AsksWhenLoadingNewerFilesystem_AnswerYes) {
     EXPECT_CALL(*console, askYesNo(HasSubstr("should not be opened with older versions"), false)).Times(1).WillOnce(Return(true));
 
     string version = newerVersion();
-    CreateWithVersion(version);
+    CreateWithVersion(version, version);
     EXPECT_NE(boost::none, Load());
 }
 
@@ -246,7 +297,7 @@ TEST_F(CryConfigLoaderTest, AsksWhenLoadingNewerFilesystem_AnswerNo) {
     EXPECT_CALL(*console, askYesNo(HasSubstr("should not be opened with older versions"), false)).Times(1).WillOnce(Return(false));
 
     string version = newerVersion();
-    CreateWithVersion(version);
+    CreateWithVersion(version, version);
     try {
         Load();
         EXPECT_TRUE(false); // expect throw
@@ -256,25 +307,25 @@ TEST_F(CryConfigLoaderTest, AsksWhenLoadingNewerFilesystem_AnswerNo) {
 }
 
 TEST_F(CryConfigLoaderTest, AsksWhenMigratingOlderFilesystem) {
-    EXPECT_CALL(*console, askYesNo(HasSubstr("Do you want to migrate it?"), false)).Times(1).WillOnce(Return(true));
+    EXPECT_CALL(*console, askYesNo(HasSubstr("Do you want to attempt a migration now?"), false)).Times(1).WillOnce(Return(true));
 
     string version = olderVersion();
-    CreateWithVersion(version);
+    CreateWithVersion(version, version);
     EXPECT_NE(boost::none, Load());
 }
 
 TEST_F(CryConfigLoaderTest, DoesNotAskForMigrationWhenCorrectVersion) {
-    EXPECT_CALL(*console, askYesNo(HasSubstr("Do you want to migrate it?"), false)).Times(0);
+    EXPECT_CALL(*console, askYesNo(HasSubstr("Do you want to attempt a migration now?"), _)).Times(0);
 
-    CreateWithVersion(gitversion::VersionString());
+    CreateWithVersion(gitversion::VersionString(), CryConfig::FilesystemFormatVersion);
     EXPECT_NE(boost::none, Load());
 }
 
 TEST_F(CryConfigLoaderTest, DontMigrateWhenAnsweredNo) {
-    EXPECT_CALL(*console, askYesNo(HasSubstr("Do you want to migrate it?"), false)).Times(1).WillOnce(Return(false));
+    EXPECT_CALL(*console, askYesNo(HasSubstr("Do you want to attempt a migration now?"), false)).Times(1).WillOnce(Return(false));
 
     string version = olderVersion();
-    CreateWithVersion(version);
+    CreateWithVersion(version, version);
     try {
         Load();
         EXPECT_TRUE(false); // expect throw
@@ -283,11 +334,24 @@ TEST_F(CryConfigLoaderTest, DontMigrateWhenAnsweredNo) {
     }
 }
 
+TEST_F(CryConfigLoaderTest, MyClientIdIsIndeterministic) {
+    TempFile file1(false);
+    TempFile file2(false);
+    uint32_t myClientId = loader("mypassword", true).loadOrCreate(file1.path(), false, false).value().myClientId;
+    EXPECT_NE(myClientId, loader("mypassword", true).loadOrCreate(file2.path(), false, false).value().myClientId);
+}
+
+TEST_F(CryConfigLoaderTest, MyClientIdIsLoadedCorrectly) {
+    TempFile file(false);
+    uint32_t myClientId = loader("mypassword", true).loadOrCreate(file.path(), false, false).value().myClientId;
+    EXPECT_EQ(myClientId, loader("mypassword", true).loadOrCreate(file.path(), false, false).value().myClientId);
+}
+
 TEST_F(CryConfigLoaderTest, DoesNotAskForMigrationWhenUpgradesAllowedByProgramArguments_NoninteractiveMode) {
     EXPECT_CALL(*console, askYesNo(HasSubstr("migrate"), _)).Times(0);
 
     string version = olderVersion();
-    CreateWithVersion(version);
+    CreateWithVersion(version, version);
     EXPECT_NE(boost::none, Load("mypassword", none, true, true));
 }
 
@@ -295,6 +359,6 @@ TEST_F(CryConfigLoaderTest, DoesNotAskForMigrationWhenUpgradesAllowedByProgramAr
   EXPECT_CALL(*console, askYesNo(HasSubstr("migrate"), _)).Times(0);
 
   string version = olderVersion();
-  CreateWithVersion(version);
+  CreateWithVersion(version, version);
   EXPECT_NE(boost::none, Load("mypassword", none, false, true));
 }
