@@ -10,6 +10,7 @@ public:
     MOCK_METHOD1(print, void(const std::string&));
     MOCK_METHOD2(ask, unsigned int(const std::string&, const std::vector<std::string>&));
     MOCK_METHOD2(askYesNo, bool(const std::string&, bool));
+    MOCK_METHOD1(askPassword, std::string(const std::string&));
 };
 
 ACTION_P(ChooseCipher, cipherName) {
@@ -22,7 +23,7 @@ class TestWithMockConsole {
 public:
     // Return a console that chooses a valid cryfs setting
     static std::shared_ptr<MockConsole> mockConsole() {
-        auto console = std::make_shared<MockConsole>();
+        auto console = std::make_shared<::testing::NiceMock<MockConsole>>();
         EXPECT_CALL(*console, ask(::testing::_, ::testing::_)).WillRepeatedly(ChooseCipher("aes-256-gcm"));
         EXPECT_CALL(*console, askYesNo(::testing::_, ::testing::_)).WillRepeatedly(::testing::Return(true));
         return console;
