@@ -1,6 +1,6 @@
 #include "testutils/FuseCreateAndOpenTest.h"
 
-#include "fspp/fuse/FuseErrnoException.h"
+#include "fspp/fs_interface/FuseErrnoException.h"
 
 using ::testing::WithParamInterface;
 using ::testing::Values;
@@ -17,7 +17,7 @@ INSTANTIATE_TEST_CASE_P(FuseCreateAndOpenErrorTest, FuseCreateAndOpenErrorTest,
 
 TEST_F(FuseCreateAndOpenErrorTest, ReturnNoError) {
   ReturnDoesntExistOnLstat(FILENAME);
-  EXPECT_CALL(fsimpl, createAndOpenFile(StrEq(FILENAME), _, _, _)).Times(1).WillOnce(Return(1));
+  EXPECT_CALL(*fsimpl, createAndOpenFile(StrEq(FILENAME), _, _, _)).Times(1).WillOnce(Return(1));
   //For the syscall to succeed, we also need to give an fstat implementation.
   ReturnIsFileOnFstat(1);
 
@@ -27,7 +27,7 @@ TEST_F(FuseCreateAndOpenErrorTest, ReturnNoError) {
 
 TEST_P(FuseCreateAndOpenErrorTest, ReturnError) {
   ReturnDoesntExistOnLstat(FILENAME);
-  EXPECT_CALL(fsimpl, createAndOpenFile(StrEq(FILENAME), _, _, _)).Times(1).WillOnce(Throw(FuseErrnoException(GetParam())));
+  EXPECT_CALL(*fsimpl, createAndOpenFile(StrEq(FILENAME), _, _, _)).Times(1).WillOnce(Throw(FuseErrnoException(GetParam())));
 
   int error = CreateAndOpenFileReturnError(FILENAME, O_RDONLY);
   EXPECT_EQ(GetParam(), error);
