@@ -1,15 +1,14 @@
 #include "testutils/FuseFstatTest.h"
 
-#include "fspp/fuse/FuseErrnoException.h"
+#include "fspp/fs_interface/FuseErrnoException.h"
 
 using ::testing::_;
-using ::testing::StrEq;
 using ::testing::WithParamInterface;
 using ::testing::Values;
 using ::testing::Eq;
-using ::testing::Return;
 using ::testing::Throw;
 
+
 using namespace fspp::fuse;
 
 // Cite from FUSE documentation on the fgetattr function:
@@ -19,10 +18,10 @@ using namespace fspp::fuse;
 
 class FuseFstatErrorTest: public FuseFstatTest, public WithParamInterface<int> {
 public:
-  int CreateFileAllowErrors(const TempTestFS *fs, const std::string &filename) {
+  /*unique_ref<OpenFileHandle> CreateFileAllowErrors(const TempTestFS *fs, const std::string &filename) {
     auto real_path = fs->mountDir() / filename;
-    return ::open(real_path.c_str(), O_RDWR | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-  }
+    return make_unique_ref<OpenFileHandle>(real_path.string().c_str(), O_RDWR | O_CREAT, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
+  }*/
 };
 INSTANTIATE_TEST_CASE_P(FuseFstatErrorTest, FuseFstatErrorTest, Values(EACCES, EBADF, EFAULT, ELOOP, ENAMETOOLONG, ENOENT, ENOMEM, ENOTDIR, EOVERFLOW));
 
@@ -30,7 +29,7 @@ TEST_P(FuseFstatErrorTest, ReturnedErrorCodeIsCorrect) {
   ReturnDoesntExistOnLstat(FILENAME);
   OnCreateAndOpenReturnFileDescriptor(FILENAME, 0);
 
-  EXPECT_CALL(fsimpl, fstat(Eq(0), _)).Times(1).WillOnce(Throw(FuseErrnoException(GetParam())));
+  EXPECT_CALL(*fsimpl, fstat(Eq(0), _)).Times(1).WillOnce(Throw(FuseErrnoException(GetParam())));
 
   auto fs = TestFS();
 
