@@ -2,7 +2,6 @@
 
 using ::testing::_;
 using ::testing::StrEq;
-using ::testing::Return;
 using ::testing::WithParamInterface;
 using ::testing::Values;
 
@@ -13,7 +12,7 @@ INSTANTIATE_TEST_CASE_P(FuseMkdirModeTest, FuseMkdirModeTest, Values(0, S_IRUSR,
 
 TEST_P(FuseMkdirModeTest, Mkdir) {
   ReturnDoesntExistOnLstat(DIRNAME);
-  EXPECT_CALL(fsimpl, mkdir(StrEq(DIRNAME), GetParam(), _, _))
+  EXPECT_CALL(*fsimpl, mkdir(StrEq(DIRNAME), GetParam(), _, _))
     .Times(1).WillOnce(FromNowOnReturnIsDirOnLstat());
 
   Mkdir(DIRNAME, GetParam());
