@@ -10,7 +10,7 @@ public:
 
 TEST_F(FuseOpenFilenameTest, OpenFile) {
   ReturnIsFileOnLstat("/myfile");
-  EXPECT_CALL(fsimpl, openFile(StrEq("/myfile"), _))
+  EXPECT_CALL(*fsimpl, openFile(StrEq("/myfile"), _))
     .Times(1).WillOnce(Return(0));
 
   OpenFile("/myfile", O_RDONLY);
@@ -19,7 +19,7 @@ TEST_F(FuseOpenFilenameTest, OpenFile) {
 TEST_F(FuseOpenFilenameTest, OpenFileNested) {
   ReturnIsDirOnLstat("/mydir");
   ReturnIsFileOnLstat("/mydir/myfile");
-  EXPECT_CALL(fsimpl, openFile(StrEq("/mydir/myfile"), _))
+  EXPECT_CALL(*fsimpl, openFile(StrEq("/mydir/myfile"), _))
     .Times(1).WillOnce(Return(0));
 
   OpenFile("/mydir/myfile", O_RDONLY);
@@ -29,7 +29,7 @@ TEST_F(FuseOpenFilenameTest, OpenFileNested2) {
   ReturnIsDirOnLstat("/mydir");
   ReturnIsDirOnLstat("/mydir/mydir2");
   ReturnIsFileOnLstat("/mydir/mydir2/myfile");
-  EXPECT_CALL(fsimpl, openFile(StrEq("/mydir/mydir2/myfile"), _))
+  EXPECT_CALL(*fsimpl, openFile(StrEq("/mydir/mydir2/myfile"), _))
     .Times(1).WillOnce(Return(0));
 
   OpenFile("/mydir/mydir2/myfile", O_RDONLY);
