@@ -3,15 +3,16 @@
 #define MESSMER_FSPP_TEST_FUSE_OPENFILE_TESTUTILS_FUSEOPENTEST_H_
 
 #include "../../../testutils/FuseTest.h"
+#include "../../../testutils/OpenFileHandle.h"
 
 class FuseOpenTest: public FuseTest {
 public:
   const char *FILENAME = "/myfile";
 
-  int OpenFile(const char *FILENAME, int flags);
+  void OpenFile(const char *FILENAME, int flags);
   int OpenFileReturnError(const char *FILENAME, int flags);
 private:
-  int OpenFileAllowError(const char *FILENAME, int flags);
+  cpputils::unique_ref<OpenFileHandle> OpenFileAllowError(const TempTestFS *fs, const char *FILENAME, int flags);
 };
 
 MATCHER_P(OpenFlagsEq, expectedFlags, "") {
