@@ -1,15 +1,11 @@
 #include "testutils/FuseReadTest.h"
 
-#include "fspp/fuse/FuseErrnoException.h"
+#include "fspp/fs_interface/FuseErrnoException.h"
 
 using ::testing::_;
-using ::testing::StrEq;
 using ::testing::WithParamInterface;
 using ::testing::Values;
 using ::testing::Eq;
-using ::testing::Return;
-using ::testing::Invoke;
-using ::testing::Throw;
 
 using namespace fspp::fuse;
 
@@ -19,11 +15,11 @@ INSTANTIATE_TEST_CASE_P(FuseReadFileDescriptorTest, FuseReadFileDescriptorTest,
 
 
 TEST_P(FuseReadFileDescriptorTest, FileDescriptorIsCorrect) {
-  ReturnIsFileOnLstatWithSize(FILENAME, 1);
+  ReturnIsFileOnLstatWithSize(FILENAME, fspp::num_bytes_t(1));
   OnOpenReturnFileDescriptor(FILENAME, GetParam());
-  EXPECT_CALL(fsimpl, read(Eq(GetParam()), _, _, _))
+  EXPECT_CALL(*fsimpl, read(Eq(GetParam()), _, _, _))
     .Times(1).WillOnce(ReturnSuccessfulRead);
 
-  char buf[1];
-  ReadFile(FILENAME, buf, 1, 0);
+  std::array<char, 1> buf{};
+  ReadFile(FILENAME, buf.data(), fspp::num_bytes_t(1), fspp::num_bytes_t(0));
 }
