@@ -1,10 +1,7 @@
 #include "testutils/FuseReadDirTest.h"
 
-using ::testing::_;
 using ::testing::StrEq;
-using ::testing::Return;
 
-using std::vector;
 using std::string;
 
 class FuseReadDirDirnameTest: public FuseReadDirTest {
@@ -12,7 +9,7 @@ public:
 };
 
 TEST_F(FuseReadDirDirnameTest, ReadRootDir) {
-  EXPECT_CALL(fsimpl, readDir(StrEq("/")))
+  EXPECT_CALL(*fsimpl, readDir(StrEq("/")))
     .Times(1).WillOnce(ReturnDirEntries({}));
 
   ReadDir("/");
@@ -20,7 +17,7 @@ TEST_F(FuseReadDirDirnameTest, ReadRootDir) {
 
 TEST_F(FuseReadDirDirnameTest, ReadDir) {
   ReturnIsDirOnLstat("/mydir");
-  EXPECT_CALL(fsimpl, readDir(StrEq("/mydir")))
+  EXPECT_CALL(*fsimpl, readDir(StrEq("/mydir")))
     .Times(1).WillOnce(ReturnDirEntries({}));
 
   ReadDir("/mydir");
@@ -29,7 +26,7 @@ TEST_F(FuseReadDirDirnameTest, ReadDir) {
 TEST_F(FuseReadDirDirnameTest, ReadDirNested) {
   ReturnIsDirOnLstat("/mydir");
   ReturnIsDirOnLstat("/mydir/mydir2");
-  EXPECT_CALL(fsimpl, readDir(StrEq("/mydir/mydir2")))
+  EXPECT_CALL(*fsimpl, readDir(StrEq("/mydir/mydir2")))
     .Times(1).WillOnce(ReturnDirEntries({}));
 
   ReadDir("/mydir/mydir2");
@@ -39,7 +36,7 @@ TEST_F(FuseReadDirDirnameTest, ReadDirNested2) {
   ReturnIsDirOnLstat("/mydir");
   ReturnIsDirOnLstat("/mydir/mydir2");
   ReturnIsDirOnLstat("/mydir/mydir2/mydir3");
-  EXPECT_CALL(fsimpl, readDir(StrEq("/mydir/mydir2/mydir3")))
+  EXPECT_CALL(*fsimpl, readDir(StrEq("/mydir/mydir2/mydir3")))
     .Times(1).WillOnce(ReturnDirEntries({}));
 
   ReadDir("/mydir/mydir2/mydir3");
